--
-- en_eventClean.sql
--
-- This procedure cleans up event logs
--
create or replace
function event_clean(job_id_in integer) return integer
-- ----------------------------------------------------------------------
-- (#) ./base/db/Oracle8i/event/en_eventClean.sql BL_core_schema_8.0.7_8.0.7.4.02_20100413  064938
-- (#) 
-- Copyright 2003 WatchMark Corp.
-- ----------------------------------------------------------------------
--
is
   default_maxLoops       constant integer := -1;   --# < 1 indicates no maximum loop
   default_maxRowsPerLoop constant integer := 1000000;
   default_cleanupAge     constant integer := 30;

   truncatePMFolderEvent wm_system_values_v.value%TYPE := NULL;
   maxLoops       integer := NULL;
   maxRowsPerLoop integer := NULL;
   cleanupAge     integer := NULL;

   UseMaxLoops    boolean := FALSE;

   loopCount   integer      := 0;
   proc_status varchar2(10) := 'OK';

   error_msg  varchar2(255);
   vComplete  boolean;
   vRowCount  integer;
   vTotalRows integer;
   vSQL       varchar2(500);

   EC_FAILURE_c CONSTANT INTEGER := 0;   -- FALSE
   EC_SUCCESS_c CONSTANT INTEGER := 1;   -- TRUE

   vReturn INTEGER := EC_SUCCESS_c;

begin

   status.update_job_status( job_id_in
                           , sched_general.TM_Message
                           , 'Event cleanup started'
                           );

   -- ---------------------------------------------------------
   -- Retrieve from WM_SYSTEM_VALUES_V if there are any
   -- defined parameters to override the defaults
   -- ---------------------------------------------------------
   --
   begin
      select max(to_number(value))
        into maxLoops
        from WM_SYSTEM_VALUES_V
       where name = 'EventCleanMaxLoops'
         and ROWNUM = 1;
   exception
   when OTHERS then NULL;
   end;

   begin
      select nvl(max(to_number(value)), default_maxRowsPerLoop)
        into maxRowsPerLoop
        from WM_SYSTEM_VALUES_V
       where name = 'EventCleanMaxRowsPerLoop'
         and ROWNUM = 1;
   exception
   when OTHERS then NULL;
   end;

   begin
      select max(to_number(value)) into cleanupAge
        from WM_SYSTEM_VALUES_V
       where name = 'EventRetention'
         and ROWNUM = 1;
   exception
   when OTHERS then NULL;
   end;

   truncatePMFolderEvent := wm_sysval.getString('EventTruncatePmFolderEventTable');

   maxLoops       := nvl(maxLoops, default_maxLoops);
   maxRowsPerLoop := nvl(maxRowsPerLoop, default_maxRowsPerLoop);
   truncatePMFolderEvent := nvl(truncatePMFolderEvent, 'Y');
   
   -- force the minimum to 1000000
   IF maxRowsPerLoop < default_maxRowsPerLoop THEN
      maxRowsPerLoop := default_maxRowsPerLoop;
   END IF;

   cleanupAge     := nvl(cleanupAge, default_cleanupAge);

   if maxLoops > 0
   then
      UseMaxLoops := TRUE;
   end if;

   -- -----------------------------------------------------------------------
   -- Truncate pm_folder_event table
   -- Before truncation we try to get a manual lock on the table,
   -- In case if some other session has the table locked then the manual lock
   -- will wait for the lock to be released from other session.
   -- 
   -- In the absense of manual lock, if some other session has a lock on the
   -- table, then the truncate statement will return Error.
   -- ------------------------------------------------------------------------ 
   IF (truncatePmFolderEvent = 'Y')
   THEN
     BEGIN
       -- vSQL := 'Truncate table pm_folder_event';
       execute immediate 'Lock table pm_folder_event in exclusive mode';  
       -- execute immediate vSQL; 

       -- Use a protected truncate
       wm_ddl.protect_truncate('pm_folder_event');
     END;
   END IF;

   -- ---------------------------------------------------------
   -- Find events that should be excluded from the cleanup
   -- and save them on a work table
   -- ---------------------------------------------------------
   --
   begin
      vSQL := 'drop table EN_CLEAN_TEMP';
      execute immediate vSQL;
   exception
   when OTHERS then NULL;
   end;

   vSQL := 'create table EN_CLEAN_TEMP nologging tablespace WM_FLEXPM '
        || 'as '
        ||       'select event_id from WM_AVAILABLE_DATA '
        || 'union select event_id from TASK_STATUS ';

   execute immediate vSQL;
   commit;

   vSQL := 'create index EN_CLEAN_TEMP_IDX1 on EN_CLEAN_TEMP (event_id) '
        || 'tablespace WM_FLEXPM';
   execute immediate vSQL;

   -- ---------------------------------------------------------
   -- main clean-up loop
   -- ---------------------------------------------------------
   --
   loopCount  := 0;
   vTotalRows := 0;

   begin

      loop

         vComplete  := TRUE;

         vRowCount := 0;

         vSQL := 'delete/*+ INDEX_FFS( e PK_EN_EVENT) */ from EN_EVENT e'
              ||  ' where handled_tstamp < sysdate - ' || cleanupAge
              ||    ' and ROWNUM <= ' || maxRowsPerLoop
              ||    ' and not exists'
              ||        ' ( select 1 from EN_CLEAN_TEMP t'
              ||          '  where t.event_id = e.event_id'
              ||        ' )'
              ;
         execute immediate vSQL;

         vRowCount := SQL%ROWCOUNT;
         vTotalRows := vTotalRows + vRowCount;
         commit;

         if (vRowCount = maxRowsPerLoop)
         then
            vComplete := FALSE;
         end if;

         loopCount := loopCount + 1;

      exit when (vComplete or (UseMaxLoops and loopCount > maxLoops));
      end loop;

      if not vComplete
      then
         proc_status := 'WARN';
      end if;


   exception
   when OTHERS then

      error_msg   := SQLERRM;
      proc_status := 'ERROR';

   end;

   if proc_status = 'OK'
   then
      status.update_job_status( job_id_in
                              , sched_general.TM_Message
                              , 'Event cleanup complete ('
                                || vTotalRows 
                                || ' rows).'
                              );
   elsif proc_status = 'WARN'
   then
      status.update_job_status( job_id_in
                              , sched_general.TM_Message
                              , 'Event cleanup finished ('
                                || vTotalRows
                                || ' rows - may not be complete)'
                              );
   else
      status.update_job_status( job_id_in
                              , sched_general.TM_Message
                              , 'Event cleanup failed :' || error_msg);

      vReturn := EC_FAILURE_c;
   end if;

   -- -----------------------------------------------------------
   -- remove work table
   -- -----------------------------------------------------------
   -- 
   begin
      vSQL := 'drop table EN_CLEAN_TEMP';
      execute immediate vSQL;
   exception
   when OTHERS then NULL;
   end;

   return vReturn;

end;
/
show errors function event_clean;

