-- ------------------------------------------------------------------------
--     COPYRIGHT WatchMark Corp 2004
--     create_schema.sql 1.1.0
-- ------------------------------------------------------------------------

-- ----------------------------------------------------------------------------------------
-- The wml_load_tables defines the meta-data used by wml_load to control what work it does
-- when WML_LOAD.DO_LOAD is called to process a particular temp table that the loader has
-- filled with traffic data.
--
-- The find_instance allows a simple subtitution of the find_instance call for this table.
-- The given find_instance call must match the following signature
--    your_package.your_find_instance(loaderKey IN VARCHAR2, entityName IN VARCHAR2) 
--      RETURN wmn_instance.instance_id%TYPE;
--
-- If you need more parameters for your find instance call, then simply define your own
-- version of the do_instance_lookup function and specify it in the instance_lookup
-- column of wml_load_tables. Obviously in this case, the find_instance call isn't
-- required unless you want to make use of it yourself.
-- 
-- Note that with the JLoader integration, wml_load_tables now supports multiple
-- configurations for a single physical table. The parameter passed in to the do_load
-- function should now be a wml_table_ref string, rather than the physical table name.
-- Existing configurations integrated with the C++ loader can simply use the table_name
-- for the wml_table_ref value.
-- ----------------------------------------------------------------------------------------
CREATE TABLE wml_load_tables (
   wml_table_ref        VARCHAR2(256) NOT NULL,
   table_name           VARCHAR2(30) NOT NULL,
   entity_name          VARCHAR2(30) NOT NULL,
   id_column_name       VARCHAR2(30) NOT NULL,
   needs_entity_map     CHAR(1) DEFAULT 'Y' CHECK (needs_entity_map    IN ('Y', 'N')) NOT NULL,
   needs_rg_reap        CHAR(1) DEFAULT 'Y' CHECK (needs_rg_reap       IN ('Y', 'N')) NOT NULL,
   needs_traf           CHAR(1) DEFAULT 'Y' CHECK (needs_traf          IN ('Y', 'N')) NOT NULL,
   needs_truncate       CHAR(1) DEFAULT 'Y' CHECK (needs_truncate      IN ('Y', 'N')) NOT NULL,
   needs_instance       CHAR(1) DEFAULT 'N' CHECK (needs_instance      IN ('Y', 'N')) NOT NULL,
   needs_trace          CHAR(1) DEFAULT 'N' CHECK (needs_trace         IN ('Y', 'N')) NOT NULL,
   needs_change_event   CHAR(1) DEFAULT 'N' CHECK (needs_change_event  IN ('Y', 'N')) NOT NULL,
   needs_last_seen      CHAR(1) DEFAULT 'N' CHECK (needs_last_seen     IN ('Y', 'N')) NOT NULL,
   find_instance_func   VARCHAR2(200),
   instance_lookup_func VARCHAR2(200),
   instance_name_colname VARCHAR2(30),
   CONSTRAINT pk_wml_load_tables PRIMARY KEY (wml_table_ref) USING INDEX TABLESPACE wm_flexpm
)
TABLESPACE wm_flexpm;



-- ----------------------------------------------------------------------------------------
-- The wml_loader_key_lookup is an optimization for mapping entity+loaderKey => instance_id
-- It it used only within wml_load. The data within the table is not critical - if it is
-- deleted, then subsequent loaders will repopulate it as new data comes in (they will just
-- be slower, since a call to find_instance is required to re-find the instance_id)
-- An IOT is very appropriate here since we are typically interested in finding all the
-- available instance_ids for a particular entity+loader_key range. Query would look
-- something like
-- SELECT instance_id
--   FROM wml_loader_key_lookup
--  WHERE entity_name = :1
--    AND loader_key BETWEEN :2 AND :3;
--
-- This translates to a nice and tight index range scan. 
-- TODO: Analyze should be updated to do a MOVE of this table periodically
-- ----------------------------------------------------------------------------------------
CREATE TABLE wml_loader_key_lookup 
(
  entity_name   VARCHAR2(30)    NOT NULL,
  loader_key    VARCHAR2(900)   NOT NULL,
  instance_id   INTEGER         NOT NULL,
  last_seen     DATE            ,
  CONSTRAINT wml_key_map_inst_fk FOREIGN KEY (instance_id) 
                                  REFERENCES wmn_instance (instance_id) ON DELETE CASCADE,
  CONSTRAINT wml_loader_key_lookup_pk PRIMARY KEY (entity_name, loader_key)
)
ORGANIZATION INDEX
TABLESPACE wm_flexpm
;

-- -----------------------------------------------
-- Suitable index to support the ON DELETE CASCADE
-- from wmn_instance should we ever need it
-- -----------------------------------------------
CREATE INDEX wml_loader_key_lookup_inst_idx 
          ON wml_loader_key_lookup(instance_id) 
  TABLESPACE wm_flexpm;


-- Register this table in wm_utility so the analyze job gets a crack at it. We want
-- to see a periodic stats collection and a MOVE.
-- SEAde67373 <robin>:
-- Doing this online move can actually cause ORA-00060 deadlocks if it happens to cross
-- paths with an INSERT, so it's best to leave well alone. The move can be done by a DBA
-- once the table size stabilizes.
-- INSERT INTO wm_utility ( table_name, attribute ) values ('WML_LOADER_KEY_LOOKUP', 'MOVE_ONLINE');
-- COMMIT;


-- --------------------------------------------------------------------------------------
-- This table is used to cache the Oracle dictionary information we need to help wml_load
-- build its dynamic queries. It gets populated in the wml_load.create_temp_tables call.
-- The reduction in logical IOs is massive, becuase we're no longer hitting the Oracle
-- dictionary views each time.
--
CREATE TABLE wml_tab_columns
(
  table_name      VARCHAR2(30) NOT NULL,
  column_name     VARCHAR2(30) NOT NULL,
  data_length     NUMBER       NOT NULL,
  is_pk           CHAR(1)      NOT NULL,
  CONSTRAINT pk_wml_tab_columns PRIMARY KEY (table_name, column_name)
)
ORGANIZATION INDEX
COMPRESS
TABLESPACE wm_flexpm
;



-- -----------------------------------------------
-- END OF FILE
-- -----------------------------------------------

