#!/usr/bin/env perl
###########################################################################
#
# Licensed Materials - Property of IBM
# 5724-T05,5724-T06
#
# (C) Copyright IBM Corp. 2003,2008. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
###########################################################################
#-- ----------------------------------------------------------------------------
#-- (#) ./base/peik/loaders/scripts/file_decode.pl BL_core_schema_8.0.7_8.0.7.4.02_20100413  064938
#-- (#) 
#-- ----------------------------------------------------------------------------
#
# Generic file_decode script. This script has two functions:
#
#  1. Option: -m -t
#     Return the instance id for a given Managed Object Name. If a File Type
#     was not provided (via the -t option), or if the File Type provided is
#     invalid, all instance id's that is associated with the given Managed
#     object name will be printed in a comma-separated line.
#  2. Option: -f
#     Parse a given file name and print out a FILEDECODE XML element.
#
# Notes:
#  + The file name passed with -f should be in the format of:
#       <ManagedObjectName>.<TimeStamp>.<FileExtension>
#  + <TimeStamp> should be in YYYYMMDDHHMI format (HH in 24 hours)
#  + <ManagedObjectName> should exist in the Managed Object List. By
#    default this is ${PROSPECT_HOME}/mo_list. An alternative list
#    can be specified with the -l option.
#
#Updated 11/02/2009 - valnt00077479
use pass_crypt;

use Getopt::Std;
use File::Basename;

my $MOList = "$ENV{'PROSPECT_HOME'}/mo_list";
my $SCRIPTNAME = basename($0);
my $STATUS = 0;

my %XMLRecord;

$XMLRecord{"filename"}      = "";
$XMLRecord{"instance_id"}   = "";
$XMLRecord{"entity_id"}     = "";
$XMLRecord{"file_type_id"}  = "";
$XMLRecord{"start_tstamp"}  = "";
$XMLRecord{"numofinterval"} = "1";

$DBPASSWD=pass_crypt::getValue($ENV{'DBUSER'});

my %FileDecode;
my %fileTypeHash;

if (!getopts('f:e:m:t:l:'))
{
   ShowUsage();
   exit 2;
}

if ( $opt_l ne '' )
{
   $MOList = $opt_l;
}

CheckMOList();
TestDBConnection();
ReadFileControl();

if ( $opt_f ne '')
{
   $XMLRecord{"filename"} = EscapeXML($opt_f);

   $basefilename = basename($XMLRecord{"filename"});
   chomp($basefilename);

   %FileDecode = SplitFileName($basefilename);
   $FullKey = GetFullKey($FileDecode{"mo_name"});
   $XMLRecord{"start_tstamp"} = GetTstamp($FileDecode{"tstamp"});

   %FileControl = GetFileControlRecord($FileDecode{"file_extension"});
   if ($FileControl{"file_type_id"} ne "")
   {
      $XMLRecord{"file_type_id"} = $FileControl{"file_type_id"};
      $XMLRecord{"entity_id"}    = $FileControl{"entity_id"};
      $XMLRecord{"instance_id"}  = GetInstanceID($FullKey,$XMLRecord{"entity_id"});
   }
   else
   {
      $XMLRecord{"file_type_id"} = "";
      $XMLRecord{"entity_id"}    = "";
      $XMLRecord{"instance_id"}  = "";
   }

   PrintXML();
}
elsif ( $opt_m ne '' || $opt_e ne '' )
{
   my $MO_id = '';

   # opt_e is for backward compat
   #
   if ( $opt_e ne '' )
   {
      $MO_id    = $opt_e;
   }
   if ( $opt_m ne '' )
   {
      $MO_id    = $opt_m;
   }
   my $FileType = $opt_t;

   chomp($MO_id);
   chomp($FileType);

   $FullKey = GetFullKey($MO_id);
   %FileControl = SelectFileControl($FileType);

   my $InstanceID = GetInstanceID($FullKey,$FileControl{"entity_id"});

   if ($InstanceID ne "")
   {
     print "$InstanceID\n";
   }
   
}
else
{
   ShowUsage();
}

exit $STATUS;


#------------------------------------------------------------------------------
sub ShowUsage()
#------------------------------------------------------------------------------
{
   print STDERR "USAGE:\n\n";
   print STDERR "   $SCRIPTNAME -f <Filename> [ -l <MOListName> ]\n";
   print STDERR "   $SCRIPTNAME -m <MOName> [ -t <FileType> ][ -l <MOListName> ]\n\n";
   print STDERR "WHERE:\n\n";
   print STDERR "  -f <FileName> : \n";
   print STDERR "        Decode file given by filename <FileName>, where <FileName>\n";
   print STDERR "        is in the format of <MOName>.<Timestamp>.<FileExtension>\n";
   print STDERR "        Timestamp should be in the format of YYYYMMDDHHMI,\n";
   print STDERR "        with HH being in a 24-hour format (0-23).\n\n";
   print STDERR "  -m <MOName> [-t <FileType>] :\n";
   print STDERR "        Find instance ID of Managed Object <MOName> and \n";
   print STDERR "        file type <FileType> if given. If -t is omitted\n";
   print STDERR "        multiple instance id's may be retrieved, and in\n";
   print STDERR "        this case, all instance id's will be printed out\n";
   print STDERR "        in a comma separated line.\n\n";
   print STDERR "  -l <MOListName> :\n";
   print STDERR "       Use file <MOListName> as the Managed Object list\n";
   print STDERR "       instead of the default \$PROSPECT_HOME/mo_list.\n\n";
}

#------------------------------------------------------------------------------
sub CheckMOList()
#------------------------------------------------------------------------------
{
   if ( ! -r $MOList)
   {
      die "Error : Cannot read $MOList";
   }
}

#------------------------------------------------------------------------------
sub TestDBConnection()
#------------------------------------------------------------------------------
{
   my $TestStatus;
	 my $TestConnect=`
	 sqlplus -s /NOLOG <<EOF | grep -v "^ *\$"
    	connect \${DBUSER}/\${DBPASSWD}@\${DBSID}
	 		exit 0;
   EOF
`;
   $TestStatus=$?;
   if ( $TestStatus ne 0 || $TestConnect =~ /SP2/)
   {
      die "Error : $TestConnect";
   }
}

#------------------------------------------------------------------------------
sub ReadFileControl()
#------------------------------------------------------------------------------
{
	my %SQLResult =`
	sqlplus -s /NOLOG <<EOF | grep -v "^ *\$"
      connect \${DBUSER}/\${DBPASSWD}@\${DBSID}
	 		set feedback off
      set head off
      whenever SQLERROR exit 1
      select upper(file_type)||','||file_type_id||','||
             file_type||','||file_extension||','||entity_id 
        from WM_FILE_CONTROL
           ;
   EOF
`;

   my @alist;

   foreach $slist (%SQLResult)
   {
      chomp($slist);
      if ($slist =~ /^ORA-/)
      {
         die "Error : $slist";
      }
      elsif ($slist ne "Connected.")
      {
         @alist = split(",",$slist);
         $fileTypeHash{$alist[0]}="$slist";
      }

   }
}

#------------------------------------------------------------------------------
sub GetFullKey($)
#------------------------------------------------------------------------------
{
   my $MOName = shift;

   my $MOLine = `grep -i "^$MOName\[ 	\]" $MOList | head -1`;
   my @MOLineArr = split(" ", $MOLine);

   my $returnKey = "$MOLineArr[1]-$MOLineArr[2]";

   if ( $returnKey eq "-" )
   {
      print STDERR "Error : $MOName not found in $MOList.\n";
      $STATUS = 9;
      $returnKey = "";
   }

   return $returnKey;
}

#------------------------------------------------------------------------------
sub SplitFileName($)
#------------------------------------------------------------------------------
{
   my $fname = shift;
   my @fnameArr = split("\\.", $fname);
   my %FDecode;

   $FDecode{"mo_name"}        = @fnameArr[0];
   $FDecode{"tstamp"}         = @fnameArr[1];
   $FDecode{"file_extension"} = ".@fnameArr[2]";

   return %FDecode;
}

#------------------------------------------------------------------------------
sub SelectFileControl($)
#------------------------------------------------------------------------------
{
   my $FType = shift;
   my %FControl;

   $FType =~ tr/a-z/A-Z/;
   %FControl = ();

   @alist = split(",", $fileTypeHash{$FType});
   $FControl{"file_type_id"}   = @alist[1];
   $FControl{"file_type"}      = @alist[2];
   $FControl{"file_extension"} = @alist[3];
   $FControl{"entity_id"}      = @alist[4];

   return %FControl;
}

#------------------------------------------------------------------------------
sub GetFileControlRecord($)
#------------------------------------------------------------------------------
{
  my $FExtension = shift;
  my %FControl;

  foreach $index (keys(%fileTypeHash))
  {

      %FControl = SelectFileControl($index);

      $val =  $FControl{"file_extension"};
      chomp($val);
      
      if ($val ne '' && $FExtension =~ m/($val)/ )
      {
         #print STDERR "$basefilename matches $val\n";
         #print STDERR "FileType is $FControl{'file_type'}\n";

         my $result = $FControl{"file_type_id"};

         if ( $result =~ /[0-9]+/)
         {
            $file_type_id = $&; 
            return %FControl;
         }
      }
      else
      {
        %FControl = ();
      }
  }
  print STDERR "Error : No matching record found for extension $FExtension\n";
  $STATUS = 9;
  return %FControl;
}


#------------------------------------------------------------------------------
sub PrintXML()
#------------------------------------------------------------------------------
{
   print "<FILEDECODE Filename=\"$XMLRecord{'filename'}\">\n";
   print "  <ATTRIBUTE Name=\"instance_id\" Value=\"$XMLRecord{'instance_id'}\"/>\n";
   print "  <ATTRIBUTE Name=\"entity_id\" Value=\"$XMLRecord{'entity_id'}\"/>\n";
   print "  <ATTRIBUTE Name=\"file_type_id\" Value=\"$XMLRecord{'file_type_id'}\"/>\n";
   print "  <ATTRIBUTE Name=\"start_tstamp\" Value=\"$XMLRecord{'start_tstamp'}\"/>\n";
   print "  <ATTRIBUTE Name=\"numofinterval\" Value=\"$XMLRecord{'numofinterval'}\"/>\n";
   print "</FILEDECODE>\n";
} 

#------------------------------------------------------------------------------
sub EscapeXML($)
#------------------------------------------------------------------------------
{
   my $string = shift;
				
   $string =~ s/&/&amp;/g;
   $string =~ s/"/&quot;/g;
   $string =~ s/'/&apos;/g;
   $string =~ s/>/&gt;/g;
   $string =~ s/</&lt;/g;

   return $string;
}

#------------------------------------------------------------------------------
sub GetInstanceID($$)
#------------------------------------------------------------------------------
{
   my $FKey = shift;
   my $EntID = shift;
   my $returnInstanceID;

   if ( $EntID eq '' )
   {
      $EntID = 'NULL';
   }

   %SQLResult =`
	 sqlplus -s /NOLOG <<! | egrep -v "^ *\$|^Connected."
      connect \${DBUSER}/\${DBPASSWD}@\${DBSID}
      set feedback off
      set head off
      set serveroutput on
      declare
         v_generic_context number;
         v_system_entity   number;
         v_instance        varchar2(100);

         cursor c_instance( p_context WMN_CONTEXT.context_id%TYPE
                          , p_system_entity WMN_ENTITY.entity_id%TYPE
                          )
         is
         select instance_id
           from WMN_INSTANCE_CONTEXT c
              , (
                   select distinct entity_id
                     from WMN_ENTITY_CONTEXT
                    where parent_entity_id = p_system_entity
                    start
                     with entity_id = nvl($EntID,entity_id)
                      and context_id = p_context
                  connect
                       by entity_id = prior parent_entity_id
                      and prior entity_id <> p_system_entity
                      and context_id = p_context
                ) b
          where c.entity_id = b.entity_id
            and c.fullkey = '$FKey'
            and c.context_id = p_context
              ;
      begin

         begin
            select entity_id
              into v_system_entity
              from WMN_ENTITY
             where upper(entity_name) ='SYSTEM'
               and rownum = 1;
         exception
         when NO_DATA_FOUND then
              v_system_entity := 1;
         end;

         select context_id
           into v_generic_context
           from WMN_CONTEXT
          where context_name like '\%Generic\%';

         v_instance := NULL;
         for r_instance in c_instance(v_generic_context, v_system_entity)
         loop
            if v_instance is not NULL
            then
               v_instance := v_instance || ',';
            end if;
            v_instance := v_instance || r_instance.instance_id;
         end loop;

         dbms_output.put_line(v_instance);

      exception
      when NO_DATA_FOUND then
           NULL;
      end;
-- don't indent slash --
/
!
`;

   my @alist;

   foreach $slist (%SQLResult)
   {
      chomp($slist);
      if ($slist =~ /^ORA-/)
      {
         print STDERR "Error : Cannot retrieve Instance ID\n";
         print STDERR "Error : $slist\n";
         $STATUS = 9;
         return "";
      } 
      elsif ($slist =~ /[a-zA-Z]+/)
      {
         $returnInstanceID = "";
      }
      elsif ($slist =~ /[0-9,]+/)
      {
         $returnInstanceID = $&;
      }
   }

   if ($returnInstanceID eq "")
   {
      print STDERR "Error : Instance ID not found.\n";
      $STATUS = 9;
   }

   return $returnInstanceID;
}

#------------------------------------------------------------------------------
sub GetTstamp($)
#------------------------------------------------------------------------------
{
   my $rawTStamp = shift;
   my $returnDate;

   my $year   = substr($rawTStamp,  0, 4);
   my $month  = substr($rawTStamp,  4, 2);
   my $day    = substr($rawTStamp,  6, 2);
   my $hour   = substr($rawTStamp,  8, 2);
   my $minute = substr($rawTStamp, 10, 2);

   if ( $year   =~ /[1-9][0-9][0-9][0-9]/ &&
        ($month =~ /0[1-9]/     || $month =~ /1[0-2]/) &&
        ($day   =~ /[0-2][0-9]/ || $day   =~ /3[0-1]/) &&
        ($hour  =~ /[0-1][0-9]/ || $hour  =~ /2[0-3]/) &&
        $minute =~ /[0-5][0-9]/ 
      )
   {
      $returnDate = "$month/$day/$year $hour\:$minute";
   }
   else
   {
      print STDERR "Error : Invalid Timestamp.\n";
      $STATUS = 9;
      $returnDate = "";
   }

   return $returnDate;
}

#
#==============================================================================
#
