/*
 */

package com.watchmark.loader.zipstream;

import java.io.FilterInputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.EOFException;
import java.io.PushbackInputStream;

/**
 * This class implements an input stream filter for reading files in the
 * cpio file format.
 *
 * @author	Robin Arnold
 */
public class CpioInputStream extends FilterInputStream
{
    private CpioEntry _entry;
    private long _remaining;
    private int _padRemaining = 0;
    private byte[] _tmpbuf = new byte[512];
    
    private boolean _closed = false;
    // this flag is set to true after EOF has reached for
    // one entry
    private boolean _entryEOF = false;
    
    /**
     * Check to make sure that this stream has not been closed
     */
    private void ensureOpen() throws IOException 
    {
    	if (_closed) 
      {
  	    throw new IOException("Stream closed");
      }
    }

    /**
     * Creates a new cpio input stream.
     * @param in the actual input stream
     */
    public CpioInputStream(InputStream ins) 
    {
      super(ins);
      if (in == null) 
      {
          throw new NullPointerException("in is null");
      }
    }

    /**
     * Reads the next cpio file entry and positions stream at the beginning
     * of the entry data.
     * @return the CpioEntry just read
     * @exception IOException if an I/O error has occurred
     */
    public CpioEntry getNextEntry() throws IOException 
    {
        ensureOpen();
        if (_entry != null) 
        {
    	    closeEntry();
      	}
        
      	if ((_entry = readHeader()) == null) 
        {
          return null;
      	}

        _remaining = _entry.getFilesize();
        _entryEOF = false;
      	return _entry;
    }

    /**
     * Closes the current cpio entry and positions the stream for reading the
     * next entry.
     * @exception IOException if an I/O error has occurred
     */
    public void closeEntry() throws IOException 
    {
        ensureOpen();
      	while (read(_tmpbuf, 0, _tmpbuf.length) != -1);
        _entryEOF = true;
    }

    /**
     * Returns 0 after EOF has reached for the current entry data,
     * otherwise always return 1.
     * <p>
     * Programs should not count on this method to return the actual number
     * of bytes that could be read without blocking.
     *
     * @return     1 before EOF and 0 after EOF has reached for current entry.
     * @exception  IOException  if an I/O error occurs.
     * 
     */
    public int available() throws IOException 
    {
        ensureOpen();
        if (_entryEOF) 
        {
            return 0;
        } 
        else 
        {
            return 1;
        }
    }

    /**
     * Reads from the current cpio entry into an array of bytes. Blocks until
     * some input is available.
     * @param b the buffer into which the data is read
     * @param off the start offset of the data
     * @param len the maximum number of bytes read
     * @return the actual number of bytes read, or -1 if the end of the
     *         entry is reached
     * @exception IOException if an I/O error has occurred
     */
    public int read(byte[] b, int off, int len) throws IOException 
    {
      ensureOpen();
      if (off < 0 || len < 0 || off > b.length - len) 
      {
        throw new IndexOutOfBoundsException();
      } 
      else if (len == 0) 
      {
        return 0;
      }

      if (_entry == null) 
      {
        return -1;
      }
        
	    if (_remaining <= 0) 
      {
        _entryEOF = true;
    		_entry = null;
        return -1;
	    }

      // Don't read more bytes that are remaining for the particular entry
	    if (len > _remaining) 
        len = (int)_remaining;
      
	    len = in.read(b, off, len);
	    if (len == -1) 
      {
        throw new IOException("unexpected EOF");
	    }
	    _remaining -= len;
	    return len;
    }

    /**
     * Skips specified number of bytes in the current cpio entry.
     * @param n the number of bytes to skip
     * @return the actual number of bytes skipped
     * @exception IOException if an I/O error has occurred
     * @exception IllegalArgumentException if n < 0
     */
    public long skip(long n) throws IOException 
    {
      if (n < 0) 
      {
          throw new IllegalArgumentException("negative skip length");
      }
      ensureOpen();
    	int max = (int)Math.min(n, Integer.MAX_VALUE);
      int total = 0;
      while (total < max) 
      {
  	    int len = max - total;
        if (len > _tmpbuf.length) 
        {
      		len = _tmpbuf.length;
        }
        len = read(_tmpbuf, 0, len);
  	    if (len == -1) 
        {
          _entryEOF = true;
          break;
        }
        total += len;
      }
      return total;
    }

    /**
     * Closes the cpio input stream.
     * @exception IOException if an I/O error has occurred
     */
    public void close() throws IOException 
    {
      if (!_closed) 
      {
        in.close();
        _closed = true;
      }
    }

    private byte[] b = new byte[256];

    /*
     * Reads the cpio header for next entry.
     */
    private CpioEntry readHeader() throws IOException 
    {
      String cpioType = "";

      // Soak up any padding so we align the start of the new header on a
      // nice boundary.
      if (_padRemaining > 0)
        readFully(_tmpbuf, 0, _padRemaining);

      int headerSize;
      try 
      {
        readFully(_tmpbuf, 0, CpioEntry.MAGIC_BYTES);
        cpioType = new String(_tmpbuf, 0, 6);
        if (cpioType.equals(CpioEntry.MAGIC_ODC))
          headerSize = CpioEntry.ODC_HEADER_BYTES;
        else
          headerSize = CpioEntry.C_HEADER_BYTES;

        readFully(_tmpbuf, 0, headerSize);
  	  } 
      catch (EOFException e) 
      {
        return null;
    	}

      // The only important fields we want are the size of the name and the
      // size of the file that follows the header.
      int nameSize;
      int fileSize;
      int nameBytes;
      String f;
      
      if (cpioType.equals(CpioEntry.MAGIC_ODC))
      {
        f = new String(_tmpbuf, CpioEntry.ODC_NAMESIZE_OFFSET, CpioEntry.ODC_NAMESIZE_LENGTH);
        nameSize = Integer.parseInt(f, 8); // 6-digit octal string
        f = new String(_tmpbuf, CpioEntry.ODC_FILESIZE_OFFSET, CpioEntry.ODC_FILESIZE_LENGTH);
        fileSize = Integer.parseInt(f, 8);

        // There is no alignment needed for 070707 ODC format, despite what the
        // docs say.
        nameBytes = nameSize;
        _padRemaining = 0;
      }
      else
      {
        // -c ASCII type. 8-bytes HEX values in ASCII.
        f = new String(_tmpbuf, CpioEntry.C_NAMESIZE_OFFSET, CpioEntry.C_NAMESIZE_LENGTH);
        nameSize = Integer.parseInt(f, 16);
        f = new String(_tmpbuf, CpioEntry.C_FILESIZE_OFFSET, CpioEntry.C_FILESIZE_LENGTH);
        fileSize = Integer.parseInt(f, 16);

        // Names are padded to a 4-byte boundary, as is the beginning of the
        // next entry header, if there is one.
        nameBytes = nameSize + pad(CpioEntry.MAGIC_BYTES + nameSize, 4);
        _padRemaining = pad(fileSize, 4);
      }
      
      // Reallocate the buffer for the filename if it's not big enough.     
      int blen = b.length;
      if (nameBytes > blen) 
      {
          do 
              blen = blen * 2;
          while (nameBytes > blen); 
          b = new byte[blen];
      } 

      readFully(b, 0, nameBytes);
      
      // Name size includes the null terminator. Remember that the real name
      // length here is nameSize not nameBytes!
      String name = new String(b, 0, nameSize);
      
      if (fileSize == 0 && name.substring(0, 10).equals("TRAILER!!!"))
        return null;

      CpioEntry e = createCpioEntry(name);
      e.setFilesize(fileSize);

    	return e;
    }
    
    private int pad(int size, int pad)
    {
      return (pad - (size % pad)) % pad;
    }


    /**
     * Creates a new <code>CpioEntry</code> object for the specified
     * entry name.
     *
     * @param name the Cpio file entry name
     * @return the CpioEntry just created
     */
    protected CpioEntry createCpioEntry(String name) 
    {
    	return new CpioEntry(name);
    }

    /**
   * Make sure we read at least len, but not more than len, bytes into the
   * buffer. This is used for reading the header (up to, but not including,
   * the header filename).
   * string.
   * @param b
   * @param off
   * @param len
   * @throws java.io.IOException
   */
    private void readFully(byte[] b, int off, int len) throws IOException 
    {
    	while (len > 0) 
      {
        int n = in.read(b, off, len);
  	    if (n == -1) 
        {
      		throw new EOFException();
        }
        off += n;
        len -= n;
    	}
    }
}
