/*
 *=============================================================================
 *                      COPYRIGHT WatchMark Inc 2004
 *=============================================================================
 * 
 * Created on Feb 23, 2004
 * 
 */
package com.watchmark.loader.framework;
import java.io.PrintWriter;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.apache.regexp.*;

public class Message
{
  private final String SEP = ",";
  
  private static final SimpleDateFormat _iso = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
  private static final SimpleDateFormat _gateway = new SimpleDateFormat("MM/dd/yy HH:mm");

  private MessageName _messageName;
  
  private List _fields = new ArrayList();
  private Map _fieldMap = new HashMap();
  private Map _keyMap = new HashMap();
  private List _tables = new ArrayList();
  private List _translators = new ArrayList();
  
  public Message(MessageName name)
  {
    _messageName = name;
  }
  
  public String getMessageNameString()
  {
    return _messageName.getName();
  }
  
  public MessageName getMessageName()
  {
    return _messageName;
  }
  
  public void addField(String fieldName)
  {
    // Record the ordinal of the field so we can do easy lookups on it
    // later.
    MessageField mf = new MessageField(fieldName, _fieldMap.size()+1);
    _fields.add(mf);
    _fieldMap.put(fieldName, mf);
  }

  public void addKey(String fieldName)
  {
    // Record the ordinal of the field so we can do easy lookups on it
    // later. Keys always start from field ordinal 0, which is the tstamp
    // column.
    MessageField mf = new MessageField(fieldName, _keyMap.size());
    _fields.add(mf);
    _keyMap.put(fieldName, mf);
  }
  
  public Iterator getFieldIterator()
  {
    return _fields.iterator();
  }
  
  public int findFieldOrdinal(String fieldName)
  {
    int result = -1;
    MessageField mf = (MessageField)_fieldMap.get(fieldName);
    if (mf != null)
      result = mf.getOrdinal();
      
    return result;
  }
  
  public int findKeyOrdinal(String keyName)
  {
    int result = -1;
    MessageField mf = (MessageField)_keyMap.get(keyName);
    if (mf != null)
      result = mf.getOrdinal();
      
    return result;
  }
  
  public MessageTable addTable(String name)
  {
    MessageTable result = new MessageTable(name);
    _tables.add(result);
    
    return result;
  }
  
  public Iterator getTableIterator()
  {
    return _tables.iterator();
  }
  
  public int getKeyCount()
  {
    // Discount F0/tstamp from our count of keys. This aligns us with the
    // field count (which skips F0).
    return _keyMap.size()-1;
  }

  public int getPackageFieldCount()
  {
    return _fieldMap.size();
  }

  public String translate(MessageTableColumn c)
  {
    String result = c.getExpression();
    
    // [Key.tstamp] => Key.F0
    // [Key.mcuID] => Key.F1
    // [ActUserHist1] => F42
    try
    {
      RE exp = new RE("\\[[A-Za-z_][A-Za-z0-9_]*\\]");
      RE key = new RE("\\[Key\\.[A-Za-z_][A-Za-z0-9_]*\\]");
      
      while (exp.match(result))
      {
        int start = exp.getParenStart(0);
        int end = exp.getParenEnd(0);
        
        String fieldName = result.substring(start+1, end-1);
        
        // Now try and find that field...
        int ordinal = findFieldOrdinal(fieldName);
        if (ordinal >= 0)
          result = result.replaceFirst("\\[" + fieldName + "\\]", "f" + ordinal);
      }

      while (key.match(result))
      {
        int start = key.getParenStart(0);
        int end = key.getParenEnd(0);
        
        String fieldName = result.substring(start+5, end-1);
        
        // Now try and find that field...
        int ordinal = findKeyOrdinal(fieldName);
        if (ordinal >= 0)
          result = result.replaceFirst("\\[Key." + fieldName + "\\]", "Key.f" + ordinal);
      }

    }
    catch (RESyntaxException e)
    {
      System.err.println("PROGRAM ERROR! Bad RE: " + e.getMessage());
    }

    return result;
  }
  
  public void addTranslator(String name, Translator t, List args)
  {
    _translators.add(new FieldTranslator(name, t, args));
  }
  
  public void runTranslators(Map attributes)
  {
    Iterator it = _translators.iterator();
    while (it.hasNext())
    {
      FieldTranslator ft = (FieldTranslator)it.next();
      ft.translate(attributes);
    }
  }

  /**
   * Ideally this should be in a separate MessageWriter class so we can
   * refine it if necessary
   * @param out
   * @param attributes
   */
  public void write(PrintWriter out, Map attributes)
  {
    // Before we go any further, run any translators we have on the attribute
    // map.
    runTranslators(attributes);
    
    // What about the tstamp? Where do we get that from?
    
    String tstamp = (String)attributes.get("tstamp");
    if (tstamp != null)
    {
      out.print(getMessageNameString());

      String tstampString = timeToGateway(tstamp);
      
      Iterator it = getFieldIterator();
      while (it.hasNext())
      {
        MessageField mf = (MessageField)it.next();
        String value;
        if (mf.getFieldName().equals("tstamp"))
          value = tstampString;
        else
          value = (String)attributes.get(mf.getFieldName());

        out.print(SEP); // always write a separator.
        if (value != null)
          out.print(value);
      }
      out.println();
    }
  }

  public static String timeToISO(Date d)
  {
    return _iso.format(d);
  }

  public static String timeToGateway(Date d)
  {
    String result = "";

    // Horrible Loader requires this nasty date format. I'm not
    // impressed.
    result = _gateway.format(d);

    return result;
  }
  
  public static String timeToGateway(String iso)
  {
    String result = "";

    try
    {
      // Horrible Loader requires this nasty date format. I'm not
      // impressed.
      Date d = _iso.parse(iso);
      result = timeToGateway(d);
    }
    catch (ParseException pe)
    {
    }
    return result;
  }
}

