package com.watchmark.loader.zipstream;
// -c zipstream.xml -o flarion.csv flarion.zip
// -c zipstream.xml -o lumts.csv lumts.zip

import com.watchmark.loader.framework.*;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Iterator;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;
import org.xml.sax.SAXException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;

public class ZipStream implements Controller
{
	private Logger _logger = Logger.getLogger(getClass());
  ArrayList _inputFiles = new ArrayList();
  
  private PrintWriter _writer = null;
//  private Writer _writer = new BufferedWriter(new OutputStreamWriter(System.out));
  
  // When we process an input file, we ask each factory if it is configured to
  // process this particular file. The first one to answer yes gets the job.
  ArrayList _fileReaders = new ArrayList();
  
  // Each file within an archive (ZIP, GZIP or TAR) has to be matched to a particular
  // processor. The first one that matches gets the job.
  ArrayList _processors = new ArrayList();

  public ZipStream()
  {
  }
  
  public void processConfigDoc(Document d)
  {
    ConfigHandler handler = new ConfigHandler(this);
    handler.process(d);
  }
  
  public void addReader(FileReader r)
  {
    _fileReaders.add(r);
  }
  
  public void addProcessor(Processor p)
  {
    _processors.add(p);
  }
  
  public void readConfigFile(String filename)
  {
    try 
    {
      DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
      DocumentBuilder builder = factory.newDocumentBuilder();
      Document document = builder.parse(filename);
      processConfigDoc(document);
    }
    catch (FactoryConfigurationError e) 
    {
      // unable to get a document builder factory
      System.err.println("Unable to get a document builder factory: " +
        e.getMessage());
    }
    catch (ParserConfigurationException e) 
    {
      // parser was unable to be configured
      System.err.println("Unable to configure parser: " +
        e.getMessage());
    }
    catch (SAXException sx) 
    {
      // parsing error
      System.err.println("XML (SAX) error: " + sx.getMessage());
      sx.printStackTrace();

      Throwable t = sx.getCause();
      if (t != null)
        t.printStackTrace();
    } 
    catch (IOException iox) 
    {
      // i/o error
      System.err.println("IO error: " + iox.getMessage());
    }
  }
  
  public void processArgs(String args[])
  {
    try
    {
      for (int i=0; i<args.length; i++)
      {
        if (args[i].equals("-c") && i+1 < args.length)
        {
          readConfigFile(args[++i]);
        }
        else if (args[i].equals("-o") && i+1 < args.length)
        {
          setWriterFile(args[++i]);
        }
        else
        {
          // all other files are treated as input.
          _inputFiles.add(args[i]);
        }
      }

      // We're writing to stdout.      
      if (_writer == null)
        _writer = new PrintWriter(new BufferedWriter(new OutputStreamWriter(System.out)));
    }
    catch (Exception e)
    {
      System.out.println(e.getMessage());
      e.printStackTrace();
    }
  }
  
  private void setWriterFile(String filename) throws Exception
  {
    // replace the stdout stream with a real file.
    _writer = new PrintWriter(new FileWriter(filename));
  }
  
  public void processFiles()
  {
    Iterator it = _inputFiles.iterator();
    while (it.hasNext())
    {
      String filename = (String)it.next();
      process(filename);
    }
    
    if (_writer != null)
    {
      _writer.flush();
      _writer.close();
    }
  }
  
  public void process(String filename)
  {
    try
    {
      FileReader r = findReader(filename);
      if (r != null)
      {
        long startTime = System.currentTimeMillis();
        r.process(filename, this);
        long endTime = System.currentTimeMillis();
        double delta = (endTime - startTime)/1000.0;

        System.err.println("ARCHIVE_TIME: " + delta + "s, " + filename);
      }
      else
        _logger.error("No reader found to accept file: " + filename);
    }
    catch (FileNotFoundException fex)
    {
      _logger.error("File not found: " + filename);
    }
    catch (IOException ioe)
    {
      _logger.error("Problem reading file: " + filename + ", " + ioe.getMessage());
      ioe.printStackTrace();
    }
  }
  
  public FileReader findReader(String filename)
  {
    FileReader result = null;

    // Create a reader from the first factory that accepts this filename.    
    Iterator it = _fileReaders.iterator();
    while (result == null && it.hasNext())
    {
      FileReader r = (FileReader)it.next();
      if (r.accept(filename))
        result = r;
    }
    
    return result;
  }

  public Processor findProcessor(String filename, String readerName)
  {
    Processor result = null;

    // Create a reader from the first factory that accepts this filename.    
    Iterator it = _processors.iterator();
    while (result == null && it.hasNext())
    {
      Processor p = (Processor)it.next();
      if (p.accept(filename, readerName))
        result = p;
    }
    
    return result;
  }
  
  public void postConfig()
  {
    // Make sure all of our processors know the writer they are supposed to
    // use.
    if (_writer != null)
    {
      Iterator it = _processors.iterator();
      while (it.hasNext())
      {
        Processor p = (Processor)it.next();
        p.setWriter(_writer);
      }
    }
  }

  /**
   * 
   * @param args
   */
  public static void main(String[] args)
  {
    Logger rootLogger = Logger.getRootLogger();
    BasicConfigurator.configure();
    
    long startTime = System.currentTimeMillis();
    ZipStream streamer = new ZipStream();
    
    streamer.processArgs(args);
    streamer.postConfig();
    streamer.processFiles();
     
    long endTime = System.currentTimeMillis();

    double delta = (endTime - startTime)/1000.0;

    System.err.println("=====================================================");
    System.err.println("Test took " + delta + "s");
    System.err.println("=====================================================");
  }
}
