-- ------------------------------------------------------------------------
-- (#) ./pweb/vendor/EnterpriseTrafficCache/agentSchema.sql BL_core_schema_8.0.7_8.0.7.4.02_20100413  064938
-- (#) 
--
-- Draft schema for WatchMark Prospect Enterprise agent activity.
-- We use tables to model a kind of scheduler for agents, where the
-- runnable queue (pe_activity_vw) changes dynamically based on
-- values in it and related tables. See this twiki reference for
-- more info:
-- http://twiki.watchmark.com/twiki_prospect/bin/view/Project/EnterpriseAgents
--
-- Robin Arnold, 2003-09-10.
--
-- ------------------------------------------------------------------------

CREATE TABLE pe_agent_types
(
  agent_type_id        INTEGER           NOT NULL,
  agent_type_name      VARCHAR2(50)      NOT NULL,
  description          VARCHAR2(64)      NOT NULL,
  max_pool_size        INTEGER           NOT NULL,
  jndi_name            VARCHAR2(256)     NOT NULL,
  max_retry_count      INTEGER           NOT NULL,
  retry_delay          INTEGER           NOT NULL,
  retry_delay_factor   FLOAT             NOT NULL,
  sched_period_minutes INTEGER           NOT NULL,  -- In minutes 
  active               CHAR(1) DEFAULT 'Y' NOT NULL,
  modify_pool_size     CHAR(1) DEFAULT 'Y' NOT NULL,
  agent_type           CHAR(1) NOT NULL, -- (L)ocal or (R)emote.
  CONSTRAINT pk_agent_types PRIMARY KEY (agent_type_id),
  CONSTRAINT chk_agent_type_schedper CHECK (sched_period_minutes >= 0),
  CONSTRAINT chk_agent_type_active CHECK (active IN ('Y', 'N')),
  CONSTRAINT chk_agent_type_canmdfy CHECK (modify_pool_size IN ('Y', 'N')),
  CONSTRAINT chk_agent_type_type CHECK (agent_type IN ('L', 'R'))
) ORGANIZATION INDEX;

CREATE UNIQUE INDEX pe_idx_agent_types_name ON pe_agent_types(agent_type_name);
CREATE SEQUENCE pe_seq_agent_types;




CREATE TABLE pe_agent_datasource
(
  agent_type_id      INTEGER         NOT NULL,
  datasource_id      INTEGER         NOT NULL,
  max_agent_count    INTEGER,
  blackout1_start    INTEGER, -- no activity during blackout period
  blackout1_end      INTEGER,
  blackout2_start    INTEGER, -- we support two blackout periods
  blackout2_end      INTEGER, -- which should be plenty of control
  active             CHAR(1) DEFAULT 'Y' NOT NULL,
  CONSTRAINT pk_agent_datasource PRIMARY KEY (agent_type_id, datasource_id),
  CONSTRAINT fk_agent_datasource_atid FOREIGN KEY (agent_type_id) 
  REFERENCES pe_agent_types (agent_type_id),
  CONSTRAINT fk_agent_datasource_dsid FOREIGN KEY (datasource_id)
  REFERENCES pe_datasource (datasource_id) ON DELETE CASCADE
) ORGANIZATION INDEX COMPRESS PCTFREE 0;




CREATE TABLE pe_activity_states
(
  state             VARCHAR2(16) NOT NULL,
  terminal          CHAR(1)      NOT NULL,
  CONSTRAINT pk_activity_states PRIMARY KEY (state),
  CONSTRAINT chk_activity_states_term CHECK (terminal IN ('Y', 'N'))
) ORGANIZATION INDEX;


-- It is useful to support grouping of activities together. This makes
-- things cleaner when it comes to scheduling activities, because a
-- group relates directly to a job, and an activity to a task. When the
-- schedule fires, the pe_agent.snap procedure is run, and changes the
-- 'fired' flag to 'Y'. This activity will then start appearing in the
-- pe_activity_vw view defined in agentSchemaViews.
--
CREATE TABLE pe_activity_group
(
  activity_group_id     INTEGER NOT NULL,
  fired                 CHAR(1) DEFAULT 'N' NOT NULL,
  CONSTRAINT pk_activity_group PRIMARY KEY (activity_group_id),
  CONSTRAINT chk_activity_group_fired CHECK (fired IN ('Y', 'N'))
) ORGANIZATION INDEX;
  
CREATE SEQUENCE pe_seq_activity_group;

-- Activities are instances of a particular kind of agent scheduled to do specific
-- task (as in piece of work, not an active_task task). This table carries denormalized
-- information from many different agent types (so many of the fields can be null). This
-- is done to minimize the join complexity, as this is the table we constantly query against
-- to see what the next piece of work (activity) is.
--
CREATE TABLE pe_activity
(
  activity_id       INTEGER           NOT NULL,
  agent_type_id     INTEGER           NOT NULL,
  activity_group_id INTEGER           NOT NULL,
  active_time       DATE              NOT NULL, -- invisible if > sysdate
  datasource_id     INTEGER, -- so where do we go to get this info?
  entity_id         INTEGER, -- entity is needed for instance and traffic agents
  origin			VARCHAR2(100), -- Set on activity creation where known.
  data_start        DATE,    -- start of window for traffic data we want
  data_end          DATE,    -- end of window
  procedure_name    VARCHAR2(61), -- procedure to call for pure PL/SQL agents
  monitor_label     VARCHAR2(100), -- Set on activity creation. Descriptive
  monitor_progress  VARCHAR2(100), -- Set by agent to indicate progress of current run
  state             VARCHAR2(16) DEFAULT 'RUNNABLE' NOT NULL,
  current_run       INTEGER, -- if RUNNING, these help us point to the latest
  current_attempt   INTEGER, -- activity_run and activity_log records
  retry             CHAR(1) DEFAULT 'N' NOT NULL, -- 'Y' when the next run is a retry
  cancel            CHAR(1) DEFAULT 'N' NOT NULL,
  ready_for_resource DATE, -- when the activity first becomes runnable. Useful for performance monitoring
  report_result_id   INTEGER, -- Used in agent to tell us report has finished
  CONSTRAINT pk_activity PRIMARY KEY (activity_id),
  CONSTRAINT chk_activity_retry CHECK (retry IN ('Y', 'N')),
  CONSTRAINT chk_activity_cancel CHECK (cancel IN ('Y', 'N')),
  CONSTRAINT fk_activity_state FOREIGN KEY (state) REFERENCES pe_activity_states(state),
  CONSTRAINT fk_activity_group FOREIGN KEY (activity_group_id) REFERENCES pe_activity_group(activity_group_id),
  CONSTRAINT fk_activity_agent_type FOREIGN KEY (agent_type_id) 
  REFERENCES pe_agent_types(agent_type_id),
  CONSTRAINT fk_activity_dsid FOREIGN KEY (datasource_id) 
  REFERENCES pe_datasource (datasource_id) ON DELETE CASCADE,
  CONSTRAINT fk_activity_eid FOREIGN KEY (entity_id) REFERENCES pe_entity (entity_id) ON DELETE CASCADE
) ORGANIZATION INDEX;

-- SEAde44516: Eliminate deadlocking.
CREATE INDEX pe_activity_grpid_idx ON pe_activity( activity_group_id );

CREATE INDEX pe_activity_stid_idx ON pe_activity(state);
CREATE INDEX pe_activity_atid_idx ON pe_activity(agent_type_id);
CREATE INDEX pe_activity_dsid_idx ON pe_activity(datasource_id);
CREATE INDEX pe_activity_eid_idx ON pe_activity(entity_id);

CREATE SEQUENCE pe_seq_activity;

-- Support dependencies between agents using a mapping table. Then using a simple 
-- join, we can easily filter out activities that are not yet runnable because their
-- agent dependents need to finish first.
CREATE TABLE pe_agent_depend
(
  agent_type_id           INTEGER NOT NULL,
  dependent_agent_type_id INTEGER NOT NULL,
  CONSTRAINT pk_agent_depend PRIMARY KEY (agent_type_id, dependent_agent_type_id),
  CONSTRAINT fk_agent_depend_aid FOREIGN KEY (agent_type_id) REFERENCES pe_agent_types(agent_type_id) ON DELETE CASCADE,
  CONSTRAINT fk_agent_depend_daid FOREIGN KEY (dependent_agent_type_id) REFERENCES pe_agent_types(agent_type_id) ON DELETE CASCADE
) ORGANIZATION INDEX;
-- -----------------------------------------------------------------------------
-- Each time the system starts, we record it in here. All events are then
-- associated with this run of the system. Helps us track events and statistics
-- over system restarts and kills etc.
-- -----------------------------------------------------------------------------
CREATE TABLE pe_system_run
(
  system_run_id     INTEGER NOT NULL,
  start_time        DATE NOT NULL, -- when did this run start?
  last_sample_time  DATE         , -- when did we last sample stats?
  stop_time         DATE,          -- when did this system stop
  clean_stop        CHAR(1),       -- was the shutdown orderly?
  CONSTRAINT pk_system_run PRIMARY KEY (system_run_id),
  CONSTRAINT chk_system_run_clean_stop CHECK (clean_stop IN ('Y', 'N'))
);

CREATE SEQUENCE pe_seq_system_run;

-- -----------------------------------------------------------------------------
-- This table hold system constants and system-state. It only ever has a single
-- row.
-- -----------------------------------------------------------------------------
CREATE TABLE pe_system
(
  system_id             INTEGER DEFAULT 1 NOT NULL,
  max_agent_count       INTEGER,
  quiesce               CHAR(1) DEFAULT 'N' NOT NULL,
  running               CHAR(1) DEFAULT 'N' NOT NULL,
  current_system_run_id INTEGER,
  ldap_url              VARCHAR(32),
  CONSTRAINT pk_system PRIMARY KEY(system_id),
  CONSTRAINT chk_system_quiesce CHECK (quiesce IN ('Y', 'N')),
  CONSTRAINT chk_system_running CHECK (running IN ('Y', 'N')),
  CONSTRAINT fk_system_srid FOREIGN KEY (current_system_run_id) REFERENCES pe_system_run (system_run_id)
);

-- -----------------------------------------------------------------------------
-- Each time an activity is run, we record the event here so we can report on success rates, run performance, etc.
-- Need to debate whether or not these should be IOTs.
-- -----------------------------------------------------------------------------
CREATE TABLE pe_activity_run
(
  activity_id           INTEGER NOT NULL,
  run_number            INTEGER NOT NULL, -- one activity can be repeated
  attempt               INTEGER NOT NULL, -- note each attempt when retrying after failure or cancellation
  system_run_id         INTEGER NOT NULL,
  start_time            DATE,
  end_time              DATE,
  data_start            DATE,
  data_end              DATE,
  remote_query_ms       INTEGER, -- total time milli-seconds in remote query
  local_query_ms        INTEGER, -- total time milli-seconds in local query
  final_state           VARCHAR2(16), -- was this run canceled, did it fail or succeed?
  final_message_text    VARCHAR2(256), -- if it failed, then why?
  CONSTRAINT pk_activity_run       PRIMARY KEY (activity_id, run_number, attempt),
  CONSTRAINT fk_activity_run_state FOREIGN KEY (final_state) REFERENCES pe_activity_states(state),
  CONSTRAINT pk_activity_run_srid  FOREIGN KEY (system_run_id) REFERENCES pe_system_run(system_run_id) ON DELETE CASCADE,
  CONSTRAINT fk_activity_run_aid   FOREIGN KEY (activity_id) REFERENCES pe_activity(activity_id) ON DELETE CASCADE
) ORGANIZATION INDEX COMPRESS
;

CREATE INDEX pe_idx_activity_run_srid ON pe_activity_run(system_run_id);

-- -----------------------------------------------------------------------------
-- Need to establish a cleanup policy. See pe_agent.cleanup for details
-- -----------------------------------------------------------------------------
CREATE TABLE pe_activity_log
(
  activity_id           INTEGER NOT NULL,
  run_number            INTEGER NOT NULL,
  attempt               INTEGER NOT NULL,
  tstamp                DATE,
  entry_number          INTEGER NOT NULL, -- make log entries unique
  system_run_id         INTEGER NOT NULL,
  message_type          CHAR(1) NOT NULL,
  message_text          VARCHAR2(256), -- size seems reasonable to me
  CONSTRAINT pk_activity_log_run PRIMARY KEY (activity_id, run_number, attempt, tstamp, entry_number),
  CONSTRAINT pk_activity_log_srid FOREIGN KEY (system_run_id) REFERENCES pe_system_run(system_run_id) ON DELETE CASCADE,
  CONSTRAINT fk_activity_log_aid FOREIGN KEY (activity_id, run_number, attempt) 
  REFERENCES pe_activity_run(activity_id, run_number, attempt) ON DELETE CASCADE,
  CONSTRAINT chk_activity_log_mtype CHECK (message_type IN ('I', 'W', 'E', 'F')) -- Info, Warning, Error, Fatal
)  ORGANIZATION INDEX COMPRESS
;

CREATE INDEX pe_idx_activity_log_srid ON pe_activity_log(system_run_id);
CREATE SEQUENCE pe_seq_activity_log;

-- -----------------------------------------------------------------------------
-- We sample the agent queue periodically to keep track of the load and see
-- how busy on average each agent type is. We can use cubes to aggregate the
-- statistics in all the dimensions we need. Queue lengths and times are stored 
-- used using Riemann sum (see kstat(3KSTAT) for details). This technique 
-- preserves correct average value semantics when aggregating.
--
-- -----------------------------------------------------------------------------
CREATE TABLE pe_agent_load
(
  tstamp            DATE              NOT NULL,
  system_run_id     INTEGER           NOT NULL,
  sample            INTEGER           NOT NULL,
  period            INTEGER           NOT NULL,
  agent_type_id     INTEGER           NOT NULL,
  datasource_id     INTEGER, -- some agents are not associated with a remote datasource
  wait_time         INTEGER           NOT NULL,
  wait_len_time     INTEGER           NOT NULL,
  run_time          INTEGER           NOT NULL,
  run_len_time      INTEGER           NOT NULL,
  CONSTRAINT pk_agent_load PRIMARY KEY (tstamp, system_run_id, sample),
  CONSTRAINT pk_agent_load_srid FOREIGN KEY (system_run_id) REFERENCES pe_system_run(system_run_id),
  CONSTRAINT fk_agent_load_atid FOREIGN KEY (agent_type_id) REFERENCES pe_agent_types(agent_type_id)
--SEAde47898 , CONSTRAINT fk_agent_load_dsid FOREIGN KEY (datasource_id) REFERENCES pe_datasource(datasource_id) ON DELETE CASCADE
) 
ORGANIZATION INDEX COMPRESS PCTFREE 0
PARTITION BY RANGE ( tstamp )
(
  PARTITION p0 VALUES LESS THAN ( TO_DATE( '2000-01-01 00', 'YYYY-MM-DD HH24' ) )
);

-- SEAde44516: Eliminate deadlocking.
-- SEAde46012: Global index on partitioned IOT.
-- CREATE INDEX pe_agent_load_ds_id_idx ON pe_agent_load( datasource_id );

-- We use a sample number to allow us to store this data in an IOT, because
-- we want to collect samples with the same time together. This is because
-- the other real members such as datasource_id and entity_id could
-- actually be null (where they are not applicable for a certain
-- agent_type_id).
CREATE SEQUENCE pe_seq_agent_load_sample;


-- New table to hold job/task activity data  SEAde63691
CREATE TABLE pe_task_activity
(
  activity_id           INTEGER NOT NULL,
  job_id                INTEGER NOT NULL,
  task_id               INTEGER NOT NULL,
  job_type              INTEGER NOT NULL,
  CONSTRAINT pk_task_activity     PRIMARY KEY (activity_id, job_id, task_id),
  CONSTRAINT fk_task_activity_aid FOREIGN KEY (activity_id) REFERENCES pe_activity(activity_id) ON DELETE CASCADE,
  CONSTRAINT fk_task_activity_jid FOREIGN KEY (task_id, job_id) REFERENCES active_task(task_id, job_id) ON DELETE CASCADE,
  CONSTRAINT fk_task_activity_jtp FOREIGN KEY (job_type) REFERENCES sched_job_types(job_type)
) ORGANIZATION INDEX;

-- ----------------------------------------------------------------------------
--
-- Local Variables:
-- mode: pls
-- End:
