---------------------------------------------------------------------------
--
-- Licensed Materials - Property of IBM
-- 5724-T05,5724-T06
--
-- (C) Copyright IBM Corp. 2004,2008. All Rights Reserved.
--
-- US Government Users Restricted Rights - Use, duplication or
-- disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
--
---------------------------------------------------------------------------
-- (#) ./pweb/vendor/EnterpriseTrafficCache/analysis-framework/schema.sql BL_core_schema_8.0.7_8.0.7.4.02_20100413  064938
-- (#) 
--
-- ---------------------------------------------------------------------------
-- Analysis Framework Schema
-- ---------------------------------------------------------------------------


-- ----------------------------------------------------------------------------
-- Different types of datasource.
-- ----------------------------------------------------------------------------
CREATE SEQUENCE af_datasource_type_seq;
CREATE TABLE af_datasource_type
(
  datasource_type_id   INTEGER      NOT NULL,
  datasource_type_name VARCHAR2(64) NOT NULL,
  CONSTRAINT pk_af_datasource_type_id PRIMARY KEY (datasource_type_id)
)
TABLESPACE wm_flexpm
  ;

CREATE UNIQUE INDEX idx_af_datasource_type_name ON af_datasource_type(datasource_type_name)
TABLESPACE wm_flexpm;

-- ----------------------------------------------------------------------------
-- Any datasource used by analysis modules must be referenced here.
-- The resource_descriptor is multi-function: interpretation depends
-- on the datasource_type_id. In the typical case, it's just a
-- table name. The datasource_name is the 'common name' or handle 
-- used to reference the datasource. Often the same as the table_name
-- in most cases.
-- ----------------------------------------------------------------------------
CREATE SEQUENCE af_datasource_seq;
CREATE TABLE af_datasource
(
  datasource_id       INTEGER NOT NULL,
  datasource_name     VARCHAR2(64) NOT NULL,
  datasource_type_id  INTEGER NOT NULL,
  resource_descriptor VARCHAR2(256),
  resource_detail     XMLType,
  pe_table_id         INTEGER, -- not null if this table is a real enterprise tbl
  CONSTRAINT pk_af_datasource PRIMARY KEY (datasource_id),
  CONSTRAINT fk_af_datasource_dstid FOREIGN KEY (datasource_type_id) REFERENCES af_datasource_type(datasource_type_id),
  CONSTRAINT fk_af_datasource_ptid FOREIGN KEY (pe_table_id) REFERENCES pe_table(table_id)
)
TABLESPACE wm_flexpm
  ;

CREATE UNIQUE INDEX idx_af_datasource_name ON af_datasource(datasource_name)
TABLESPACE wm_flexpm;

-- ----------------------------------------------------------------------------
-- The af_datasource_category table serves to categorise datasources.
-- ----------------------------------------------------------------------------
CREATE SEQUENCE af_datasource_category_seq;
CREATE TABLE af_datasource_category
(
  datasource_category_id INTEGER NOT NULL,
  category_name          VARCHAR2(255) NOT NULL
)
TABLESPACE wm_flexpm;

ALTER TABLE af_datasource_category
  ADD CONSTRAINT pk_af_datasource_category PRIMARY KEY (datasource_category_id);
ALTER TABLE af_datasource_category
  ADD CONSTRAINT uq_af_datasource_category_name UNIQUE (category_name);

-- ----------------------------------------------------------------------------
-- af_datasource_access models how the client gets access to underlying data.
-- For instance, each datasource (e.g. table) may be accessed through several
-- different views, each perhaps doing different analysis like ranking or
-- aggregating (e.g. windowed average).
-- The jndi_name field references an implementation of a 
-- DataRepositoryAccessBean
-- The dao_class_name indicates what DAO class should be used by the repository
-- access bean to fetch the data.
-- The db_object_name identifies the table, view or function providing this
-- data.
-- access_detail is Castor based XML that the DAO uses to configure the query
-- against the object. The XML can be different for each DAO class defined
-- above (each type of DAO needs its own set of parameters).
-- ----------------------------------------------------------------------------
CREATE SEQUENCE af_datasource_access_seq;
CREATE TABLE af_datasource_access
(
  datasource_access_id    INTEGER NOT NULL,
  datasource_access_name  VARCHAR2(64) NOT NULL,
  jndi_name               VARCHAR2(64) NOT NULL,
  dao_class_name          VARCHAR2(256) NOT NULL,
  db_object_name          VARCHAR2(256) NOT NULL,
  db_object_type          CHAR(1),
  access_config           XMLType,
  chartable               CHAR(1) DEFAULT ('Y') NOT NULL,
  datasource_category_id  INTEGER, 
  kpi_name                VARCHAR2(256),
  kpi_heading1            VARCHAR2(256),
  kpi_heading2            VARCHAR2(256), 
  CONSTRAINT pk_af_datasource_access PRIMARY KEY (datasource_access_id),
  CONSTRAINT ck_af_datasource_access_chrt CHECK (chartable IN ('Y', 'N')), 
  CONSTRAINT fk_af_datasource_category FOREIGN KEY (datasource_category_id)
    REFERENCES af_datasource_category (datasource_category_id)
)
TABLESPACE wm_flexpm
;

CREATE UNIQUE INDEX idx_af_datasource_access_nm ON af_datasource_access(datasource_access_name)
TABLESPACE wm_flexpm;


-- ----------------------------------------------------------------------------
-- af_datasource_access_map connects datasource inputs to datasource_access
-- so we can represent multiple tables affecting a single view
-- ----------------------------------------------------------------------------
CREATE TABLE af_datasource_access_map
(
  datasource_access_id    INTEGER NOT NULL,
  datasource_id           INTEGER NOT NULL,
  CONSTRAINT pk_af_dsrc_acs_map PRIMARY KEY (datasource_access_id, datasource_id),
  CONSTRAINT fk_af_dsrc_acs_map_aid FOREIGN KEY (datasource_access_id)
    REFERENCES af_datasource_access(datasource_access_id) ON DELETE CASCADE,
  CONSTRAINT fk_af_dsrc_acs_map_did FOREIGN KEY (datasource_id)
    REFERENCES af_datasource(datasource_id) ON DELETE CASCADE
)
TABLESPACE wm_flexpm
;


-- ----------------------------------------------------------------------------
-- A process represents an instance of a module.
-- All analysis modules usually share the same agent_type: ANALYSIS. But we
-- support the creation of multiple analysis agent types to allow us greater
-- control over concurrencly (some may be very expensive, and so we might want
-- to limit their concurrency.
-- jndi_name is the JNDI address of the AnalysisSessionBean that implements
-- this module.
-- process_config is a container for module-specific configuration info. This
-- XML is rendered by Castor. This allows us to re-use the same basic bean
-- for potentially different tasks. 
-- Note that we don't care what the bean is actually called. All we care about
-- is that it implements the AnalysisSessionBean interface.
-- ----------------------------------------------------------------------------
CREATE SEQUENCE af_process_seq;
CREATE TABLE af_process
(
  process_id          INTEGER NOT NULL,
  process_name        VARCHAR2(30) NOT NULL,
  agent_type_id       INTEGER NOT NULL,
  jndi_name           VARCHAR2(256) NOT NULL,
  process_config      XMLType,
  CONSTRAINT pk_af_process_id PRIMARY KEY (process_id),
  CONSTRAINT fk_af_process_atid FOREIGN KEY (agent_type_id) REFERENCES pe_agent_types(agent_type_id)
)
TABLESPACE wm_flexpm
;

CREATE UNIQUE INDEX idx_af_datasource_process_name ON af_process(process_name)
TABLESPACE wm_flexpm;


-- ----------------------------------------------------------------------------
--
-- ----------------------------------------------------------------------------
CREATE SEQUENCE af_process_input_seq;
CREATE TABLE af_process_input
(
  process_input_id    INTEGER NOT NULL,
  process_id          INTEGER NOT NULL,
  af_datasource_id    INTEGER,
  CONSTRAINT pk_af_process_input_id PRIMARY KEY (process_input_id),
  CONSTRAINT fk_af_process_input_pid FOREIGN KEY (process_id) REFERENCES
    af_process(process_id) ON DELETE CASCADE,
  CONSTRAINT fk_af_process_input_tid FOREIGN KEY (af_datasource_id) REFERENCES
    af_datasource(datasource_id) ON DELETE CASCADE
)
TABLESPACE wm_flexpm;

CREATE UNIQUE INDEX idx_af_process_input ON af_process_input(process_id, af_datasource_id)
TABLESPACE wm_flexpm;

-- ----------------------------------------------------------------------------
--
-- ----------------------------------------------------------------------------
CREATE SEQUENCE af_process_output_seq;
CREATE TABLE af_process_output
(
  process_output_id   INTEGER NOT NULL,
  process_id          INTEGER NOT NULL,
  af_datasource_id    INTEGER,
  CONSTRAINT pk_af_process_output_id PRIMARY KEY (process_output_id),
  CONSTRAINT fk_af_process_output_pid FOREIGN KEY (process_id) REFERENCES
    af_process(process_id) ON DELETE CASCADE,
  CONSTRAINT fk_af_process_output_tid FOREIGN KEY (af_datasource_id) REFERENCES
    af_datasource(datasource_id) ON DELETE CASCADE
)
TABLESPACE wm_flexpm;

CREATE UNIQUE INDEX idx_af_process_output ON af_process_output(process_id, af_datasource_id)
TABLESPACE wm_flexpm;

-- ----------------------------------------------------------------------------
--
-- ----------------------------------------------------------------------------
CREATE TABLE af_activity
(
  activity_id          INTEGER NOT NULL,
  process_id           INTEGER NOT NULL,
  table_id             INTEGER,
  start_key            VARCHAR2(100),
  end_key              VARCHAR2(100),
  start_instance_id    INTEGER,
  end_instance_id      INTEGER,
  CONSTRAINT pk_af_activity PRIMARY KEY (activity_id),
  CONSTRAINT fk_af_activity_actid FOREIGN KEY (activity_id) REFERENCES pe_activity (activity_id) ON DELETE CASCADE,
  CONSTRAINT fk_af_activity_pid   FOREIGN KEY (process_id) REFERENCES af_process (process_id) ON DELETE CASCADE,
  CONSTRAINT fk_af_activity_tid   FOREIGN KEY (table_id) REFERENCES pe_table(table_id) ON DELETE CASCADE
)
TABLESPACE wm_flexpm;

-- Index in support of the cascade delete from pe_activity
CREATE INDEX idx_af_activity_actid ON af_activity (process_id)
TABLESPACE wm_flexpm;
  
CREATE INDEX idx_af_activity_tid ON af_activity (table_id)
TABLESPACE wm_flexpm;


CREATE TABLE wmd_kpi_load_stage
(
  entity_name    VARCHAR2(64),
  field_name     VARCHAR2(64),
  heading1       VARCHAR2(40),
  heading2       VARCHAR2(40),
  aggr_type      VARCHAR2(10),
  tech_types     VARCHAR2(256),
  calc_source    VARCHAR2(2000)
)
TABLESPACE wm_flexpm;


-- ----------------------------------------------------------------------------
-- Views
-- ----------------------------------------------------------------------------
CREATE OR REPLACE VIEW af_datasource_vw AS
  SELECT da.datasource_access_id,
         da.datasource_access_name,
         da.jndi_name, 
         da.dao_class_name,
         da.db_object_name,
         da.access_config,
         da.kpi_name,
         da.kpi_heading1,
         da.kpi_heading2,
         da.chartable,
         dc.category_name
    FROM af_datasource_access da, af_datasource_category dc
   WHERE da.datasource_category_id = dc.datasource_category_id (+);


