-- ------------------------------------------------------------------------
-- (#) ./pweb/vendor/EnterpriseTrafficCache/cacheSchema.sql BL_core_schema_8.0.7_8.0.7.4.02_20100413  064938
-- (#) 
--
-- Draft schema for WatchMark Prospect Enterprise. A lot of this came
-- from the original Enterprise Architecture (EA) work done by
-- Richard Sheldon.
--
-- Robin Arnold, 2003-06-17.
--
-- ------------------------------------------------------------------------


-- ------------------------------------------------------------------------
-- Enterprise instance ids come from a single source
-- ------------------------------------------------------------------------
CREATE SEQUENCE pe_seq_instance;

-- ------------------------------------------------------------------------
-- Table used to define the complete set of column types so we keep this
-- information stashed away in one place. We may want to use the type_name
-- rather than the id as FKs in other tables to aid readability.
-- ------------------------------------------------------------------------
CREATE TABLE pe_column_type
(
  column_type_id         INTEGER NOT NULL,
  type_name              VARCHAR2(30) NOT NULL,
  type_tag               CHAR(1) NOT NULL,
  type_label             VARCHAR2(256),
  data_precision         INTEGER,
  data_scale             INTEGER,
  CONSTRAINT pk_column_type PRIMARY KEY (column_type_id),
  CONSTRAINT ck_type_tag CHECK (type_tag IN ('I', 'F', 'S'))
);

CREATE UNIQUE INDEX pe_idx_column_type_name ON pe_column_type(UPPER(type_name));
CREATE SEQUENCE pe_seq_column_type;

-- ------------------------------------------------------------------------
-- Each server we are connected to is represented by a datasource row in
-- pe_datasource.
-- ------------------------------------------------------------------------
CREATE TABLE pe_datasource
(
  datasource_id          INTEGER NOT NULL,       -- normalized PK
  datasource_name      VARCHAR2(100) NOT NULL, -- familiar name
  current_tz_offset      INTEGER DEFAULT 0 NOT NULL,
  report_timeslices      INTEGER DEFAULT 0,
  busyhour_timeslices    INTEGER DEFAULT 0,
  max_allowed_agents     INTEGER DEFAULT 10,
  active                 CHAR(1) DEFAULT 'Y' NOT NULL,
  local                  CHAR(1) DEFAULT 'N' NOT NULL,
  deleted                CHAR(1) DEFAULT 'N' NOT NULL, -- logically deleted
  purge_active           CHAR(1) DEFAULT 'N', -- delete-in-progress. Cannot reuse
  hostname               VARCHAR2(100),
  fx_dataserver_port     VARCHAR2(32),
  max_fetch_hours        INTEGER DEFAULT 24, -- how many hours of traffic can we fetch
  group_security         CHAR(1) DEFAULT 'N' NOT NULL, -- group security enabled
  CONSTRAINT pk_datasource   PRIMARY KEY (datasource_id),
  CONSTRAINT chk_datasource_active       CHECK (active IN ('Y', 'N')),
  CONSTRAINT chk_datasource_local        CHECK (local  IN ('Y', 'N')),
  CONSTRAINT chk_datasource_deleted      CHECK (deleted  IN ('Y', 'N')),
  CONSTRAINT chk_datasource_purge_active CHECK (purge_active IN ('Y','N')),
  CONSTRAINT chk_datasource_group_sec    CHECK (group_security  IN ('Y', 'N'))
);

-- Prevent duplicate datasource names
CREATE UNIQUE INDEX pe_idx_datasource_name ON pe_datasource(UPPER(datasource_name));

CREATE SEQUENCE pe_seq_datasource;
     
-- -----------------------------------------------------------------------
-- contains non-specific datasource attributes.
-- -----------------------------------------------------------------------
CREATE TABLE pe_datasource_attribute
(
  datasource_id          INTEGER NOT NULL,
  attr_name              VARCHAR2(64) NOT NULL,
  attr_value             VARCHAR2(64) NOT NULL,
  CONSTRAINT pk_ds_attribute PRIMARY KEY (datasource_id, attr_name ),
  CONSTRAINT fk_ds_attribute FOREIGN KEY (datasource_id) REFERENCES pe_datasource(datasource_id) ON DELETE CASCADE
);

CREATE UNIQUE INDEX pe_idx_ds_attribute ON pe_datasource_attribute( datasource_id, UPPER( attr_name ) );


-- ------------------------------------------------------------------------
-- Module (represents a product module such as 'Lucent', or 'Motorola'. A
-- Prospect instance can support multiple modules (see module_deplyments)
-- ------------------------------------------------------------------------
CREATE TABLE pe_module
(
  module_id          INTEGER NOT NULL,
  module_name    VARCHAR2(30) NOT NULL,
  module_prefix      VARCHAR2(8) NOT NULL,
  CONSTRAINT pk_module   PRIMARY KEY (module_id)
);

CREATE UNIQUE INDEX pe_idx_module_name ON pe_module(UPPER(module_name));

CREATE SEQUENCE pe_seq_module;


-- ------------------------------------------------------------------------
-- Module Version. Represents a revision of a particular model (including
-- any patches that might have been supplied. Two systems that share the 
-- same module_version are expected to look identical in terms of their
-- data-dictionary. This means their data can share columns in the 
-- traffic cache data tables. (the expressions are switched on the mvid).
-- Note that a module_version has a hierarchy. This helps searches for
-- a suitable version when an exact match isn't available.
-- ------------------------------------------------------------------------
CREATE TABLE pe_module_version
(
  module_version_id      INTEGER NOT NULL,
  module_id              INTEGER NOT NULL,
  version_name           VARCHAR2(30) NOT NULL,
  version_prefix     VARCHAR2(8) NOT NULL,
  version_number         NUMBER(6,3) NOT NULL,
  parent_version_id      INTEGER,
  root_entity_name       VARCHAR2(100),
  CONSTRAINT pk_module_version   PRIMARY KEY (module_version_id),
  CONSTRAINT fk_module_version_mid FOREIGN KEY (module_id) REFERENCES pe_module(module_id) ON DELETE CASCADE,
  CONSTRAINT fk_module_version_pvid FOREIGN KEY (parent_version_id) REFERENCES pe_module_version (module_version_id)
);

CREATE UNIQUE INDEX pe_idx_module_version_name 
  ON pe_module_version(module_id, UPPER(version_name));

CREATE SEQUENCE pe_seq_module_version;

-- ------------------------------------------------------------------------
-- Logical subdivision of a module version used for mapping purposes
-- ------------------------------------------------------------------------
CREATE TABLE pe_module_version_group
(
  module_version_group_id         INTEGER NOT NULL,
  module_version_id               INTEGER NOT NULL,
  group_name                      VARCHAR2(30) NOT NULL,
  is_default                      CHAR(1) DEFAULT 'N' NOT NULL,
  CONSTRAINT pk_module_version_group PRIMARY KEY (module_version_group_id),
  CONSTRAINT fk_mv_group_mvid FOREIGN KEY (module_version_id) REFERENCES pe_module_version (module_version_id) ON DELETE CASCADE,
  CONSTRAINT chk_mv_group_default  CHECK (is_default IN ('Y', 'N'))
);

CREATE UNIQUE INDEX pe_idx_mv_group_name
  ON pe_module_version_group(module_version_id, UPPER(group_name));
  
CREATE SEQUENCE pe_seq_module_version_group;

-- ------------------------------------------------------------------------
-- Module Instance. Represents a particular instance of a module version
-- required because all module versions might not look exactly the same.
-- Some might have been patched or hacked, and we need to be able to
-- distinguish between them. Luckily the gateway digest gives us a really
-- good idea if two datasources are compatible (and hence can share the
-- same pe_module_instance).
-- ------------------------------------------------------------------------
CREATE TABLE pe_module_instance
(
  module_instance_id     INTEGER NOT NULL,
  module_version_id      INTEGER NOT NULL,
  digest                 VARCHAR2(256),
  created                DATE NOT NULL,
  CONSTRAINT pk_module_instance   PRIMARY KEY (module_instance_id),
  CONSTRAINT fk_module_instance_mvid FOREIGN KEY (module_version_id) REFERENCES pe_module_version(module_version_id) ON DELETE CASCADE
);

CREATE SEQUENCE pe_seq_module_instance;

-- ------------------------------------------------------------------------
-- Describes which datasources have which modules. Each datasource 
-- (remote server) can support many modules, though only one version 
-- of that module (this constraint is enforced using an additional 
-- index pe_idx_module_version_map_mds.
-- ------------------------------------------------------------------------
CREATE TABLE pe_module_deployments
(
  -- No normalized PK.
  module_instance_id    INTEGER NOT NULL,
  module_id             INTEGER NOT NULL,
  datasource_id         INTEGER NOT NULL,
  CONSTRAINT pk_module_deployments   PRIMARY KEY (module_instance_id, module_id, datasource_id),
  CONSTRAINT fk_module_deployments_miid FOREIGN KEY (module_instance_id) REFERENCES pe_module_instance(module_instance_id) ON DELETE CASCADE,
  CONSTRAINT fk_module_deployments_mid FOREIGN KEY (module_id) REFERENCES pe_module(module_id) ON DELETE CASCADE,
  CONSTRAINT fk_module_deployments_did FOREIGN KEY (datasource_id) REFERENCES pe_datasource(datasource_id) ON DELETE CASCADE
);

-- A datasource can only have one version of a particular module.
CREATE UNIQUE INDEX pe_idx_module_deployments_mds ON pe_module_deployments(module_id, datasource_id);

-- ------------------------------------------------------------------------
-- Table to hold remote system information for remote reporting imported
-- by the data dictionary agent
-- ------------------------------------------------------------------------
CREATE TABLE pe_remote_system
(
  -- No normalized PK.
  datasource_id         INTEGER NOT NULL,
  system_id             VARCHAR2(50) NOT NULL,
  CONSTRAINT pk_remote_system PRIMARY KEY(datasource_id, system_id),
  CONSTRAINT fk_remote_system_did FOREIGN KEY (datasource_id) REFERENCES pe_datasource(datasource_id) ON DELETE CASCADE
);

-- ------------------------------------------------------------------------
-- Enterprise 'entities' are things like 'Switch' and 'Cell' - they are
-- generic across vendors and platforms.
-- ------------------------------------------------------------------------
CREATE TABLE pe_entity
(
  entity_id             INTEGER NOT NULL,
  entity_name           VARCHAR2(25) NOT NULL,
  image_key             VARCHAR2(20),
  parententity_id       INTEGER,
  retention_days        INTEGER,                          -- NULL means keep forever.
  data_frequency_id     INTEGER DEFAULT 7 NOT NULL,       -- Default hourly
  is_defined            CHAR(1) DEFAULT 'N' NOT NULL,     -- Default not defined
  is_enabled            CHAR(1) DEFAULT 'N' NOT NULL,     -- Default not enabled
  CONSTRAINT pk_entity   PRIMARY KEY (entity_id),
  CONSTRAINT ck_entity_rday CHECK (retention_days >= 0),
  CONSTRAINT ck_entity_dfreq_id FOREIGN KEY(data_frequency_id) REFERENCES wm_frequency(frequency_id),
  CONSTRAINT ck_entity_idef CHECK (is_defined IN ('Y', 'N')),
  CONSTRAINT ck_entity_iena CHECK (is_enabled IN ('Y', 'N'))
);

CREATE UNIQUE INDEX pe_idx_entity_name ON pe_entity(UPPER(entity_name));

CREATE SEQUENCE pe_seq_entity;


-- ------------------------------------------------------------------------
-- Module Version Entity (mv_entity). A module version entity  
-- represents a remote collection of data. Note that entity is a 
-- Prospect term, so we use it loosely as we might also be referencing
-- data on non-Prospect systems.
-- This table represents objects like:
-- module.module_version.entity_name
-- LUC.R19.ECP
-- LUC.R20.ECP
-- NOR.MTX10.MSC
-- ------------------------------------------------------------------------
CREATE TABLE pe_mv_entity
(
  mv_entity_id                 INTEGER NOT NULL,
  module_version_id            INTEGER NOT NULL,
  entity_name                  VARCHAR2(100) NOT NULL,
  image_key                    VARCHAR2(20),
  entity_id                    INTEGER, -- set when mapped to an enterprise entity
  mv_group_id                  INTEGER, -- set when mapped to an enterprise entity
  parent_mv_entity_id          INTEGER, -- reflects the tree of the remote (traffic) dictionary
  can_report                   CHAR(1) DEFAULT 'Y' NOT NULL, -- Default can report
  CONSTRAINT pk_mv_entity      PRIMARY KEY (mv_entity_id),
  CONSTRAINT fk_mv_entity_mvid FOREIGN KEY (module_version_id) REFERENCES pe_module_version (module_version_id) ON DELETE CASCADE,
  CONSTRAINT fk_mpdule_version_entity_pid  FOREIGN KEY (entity_id) REFERENCES pe_entity (entity_id),
  CONSTRAINT fk_mv_entity_mv_group_id FOREIGN KEY (mv_group_id) REFERENCES pe_module_version_group (module_version_group_id),
  CONSTRAINT ck_mv_entity_crpt CHECK (can_report IN ('Y', 'N'))
);

CREATE UNIQUE INDEX pe_idx_mv_entity_name ON pe_mv_entity (module_version_id, UPPER(entity_name));

-- SEAde44516: Eliminate deadlocking.
CREATE INDEX pe_mv_entity_entity_id_idx ON pe_mv_entity (entity_id);

CREATE SEQUENCE pe_seq_mv_entity;


-- ------------------------------------------------------------------------
-- Temp table for inserting module version entities into pe_mv_entity
-- ------------------------------------------------------------------------
CREATE GLOBAL TEMPORARY TABLE pe_mv_entity_tmp
(
  entity_name        VARCHAR2(100) NOT NULL,
  module_version_id  INTEGER NOT NULL,
  image_key          VARCHAR2(20),
  parent_entity_name VARCHAR2(100),
  can_report         CHAR(1) NOT NULL, 
  CONSTRAINT pk_mv_entity_tmp PRIMARY KEY (entity_name)
) 
ON COMMIT DELETE ROWS;

-- ------------------------------------------------------------------------
-- 
-- 
-- 
-- 
-- ------------------------------------------------------------------------
CREATE TABLE pe_db_status
(
  status      CHAR(1) NOT NULL,
  name   VARCHAR2(20) NOT NULL,
    CONSTRAINT pk_pe_col_status PRIMARY KEY ( status )
);

CREATE UNIQUE INDEX pe_idx_db_status ON pe_db_status(UPPER(name));

-- ------------------------------------------------------------------------
-- Table to hold field group names imported from remote systems
-- ------------------------------------------------------------------------
CREATE TABLE pe_mv_field_group
(
  field_group_id    INTEGER        NOT NULL,
  field_group_name  VARCHAR2(20)   NOT NULL,
  field_group_short VARCHAR(5),
  CONSTRAINT pk_mv_field_group_id PRIMARY KEY (field_group_id)
);

CREATE UNIQUE INDEX pe_idx_mv_field_group ON pe_mv_field_group (UPPER(field_group_name));
CREATE INDEX pe_idx_mv_field_group_short ON pe_mv_field_group (UPPER(field_group_short));
CREATE SEQUENCE pe_seq_mv_field_group;

-- ------------------------------------------------------------------------
-- Table to hold imported tech types
-- ------------------------------------------------------------------------
CREATE TABLE pe_field_technology
(
   tech_id    INTEGER       NOT NULL,
   tech_name  VARCHAR2(8)   NOT NULL,
   tech_char  CHAR(1)       NULL,
   tech_desc  VARCHAR2(100) NULL,
   CONSTRAINT pk_field_technology PRIMARY KEY (tech_id) 
);

CREATE UNIQUE INDEX pe_idx_field_technology ON pe_field_technology (UPPER(tech_name));
CREATE SEQUENCE pe_seq_field_technology;

-- ------------------------------------------------------------------------
-- Table to hold imported field attribute names
-- ------------------------------------------------------------------------
CREATE TABLE pe_field_attribute
(
   attribute_id    INTEGER       NOT NULL,
   attribute_name  VARCHAR2(40)  NOT NULL,
   CONSTRAINT pk_field_attribute PRIMARY KEY (attribute_id)
);

CREATE UNIQUE INDEX pe_idx_field_attribute ON pe_field_attribute (UPPER(attribute_name));
CREATE SEQUENCE pe_seq_field_attribute;

-- ------------------------------------------------------------------------
-- Table to map module version to aggregation types
-- ------------------------------------------------------------------------
CREATE TABLE pe_mv_aggregation_map
(
    module_version_id            INTEGER NOT NULL,
    aggr_id                      CHAR(1) NOT NULL,
    CONSTRAINT pk_mv_aggr_map PRIMARY KEY (module_version_id, aggr_id),
    CONSTRAINT fk_mv_aggr_map_mvid FOREIGN KEY (module_version_id) REFERENCES pe_module_version(module_version_id) ON DELETE CASCADE,
    CONSTRAINT fk_mv_aggr_map_aggrid FOREIGN KEY (aggr_id) REFERENCES wmd_aggregator_base(aggr_id)
);

-- ------------------------------------------------------------------------
-- Table to map module version to field groups
-- ------------------------------------------------------------------------
CREATE TABLE pe_mv_field_group_map
(
    module_version_id            INTEGER NOT NULL,
    field_group_id               INTEGER NOT NULL,
    CONSTRAINT pk_mv_field_group_map PRIMARY KEY (module_version_id, field_group_id),
    CONSTRAINT fk_mv_field_group_map_mvid FOREIGN KEY (module_version_id) REFERENCES pe_module_version(module_version_id) ON DELETE CASCADE,
    CONSTRAINT fk_mv_field_group_map_fgid FOREIGN KEY (field_group_id) REFERENCES pe_mv_field_group(field_group_id)
);

-- ------------------------------------------------------------------------
-- Table to map module version to imported tech types
-- ------------------------------------------------------------------------
CREATE TABLE pe_mv_technology_map
(
    module_version_id            INTEGER NOT NULL,
    tech_id                      INTEGER NOT NULL,
    CONSTRAINT pk_mv_tech_map PRIMARY KEY (module_version_id, tech_id),
    CONSTRAINT fk_mv_tech_map_mvid FOREIGN KEY (module_version_id) REFERENCES pe_module_version(module_version_id) ON DELETE CASCADE,
    CONSTRAINT fk_mv_tech_map_techid FOREIGN KEY (tech_id) REFERENCES pe_field_technology(tech_id)
);

-- ------------------------------------------------------------------------
-- Table to map module version to imported field attribute names
-- ------------------------------------------------------------------------
CREATE TABLE pe_mv_field_attribute_map
(
    module_version_id            INTEGER NOT NULL,
    attribute_id                 INTEGER NOT NULL,
    editable                     CHAR(1) NOT NULL,
    CONSTRAINT pk_mv_field_attr_map PRIMARY KEY (module_version_id, attribute_id),
    CONSTRAINT fk_mv_field_attr_map_mvid FOREIGN KEY (module_version_id) REFERENCES pe_module_version(module_version_id) ON DELETE CASCADE,
    CONSTRAINT fk_mv_field_attr_map_attrid FOREIGN KEY (attribute_id) REFERENCES pe_field_attribute(attribute_id),
    CONSTRAINT ck_field_attribute_edit CHECK (editable IN ('Y', 'N'))
);

-- ------------------------------------------------------------------------
-- Module Version Field. This table should be populated by
-- dictionary agents. Their primary purpose is to support the Enterprise
-- Mapping tool. By storing this locally, it means the mapping tool can
-- work without having a live connection to the remote datasource (usually
-- abstracted through a WmProspectDictionary object).
-- ------------------------------------------------------------------------
CREATE TABLE pe_mv_field
(
  mv_field_id                 INTEGER NOT NULL,
  mv_entity_id                INTEGER NOT NULL,
  field_name                  VARCHAR2(128) NOT NULL,
  heading1                    VARCHAR2(40),
  heading2                    VARCHAR2(40),
  description                 VARCHAR2(1000),
  field_group_id              INTEGER,
  column_type_id              INTEGER NOT NULL,
  aggr                        CHAR(1) NOT NULL,
  is_dimension                CHAR(1) NOT NULL,
  is_virtual                  CHAR(1) NOT NULL,
  is_available                CHAR(1) NOT NULL,
  CONSTRAINT pk_mv_field_id   PRIMARY KEY (mv_field_id),
  CONSTRAINT fk_mv_field_did  FOREIGN KEY (mv_entity_id) REFERENCES pe_mv_entity (mv_entity_id) ON DELETE CASCADE,
  CONSTRAINT fk_mv_field_ctid FOREIGN KEY (column_type_id) REFERENCES pe_column_type (column_type_id),
  CONSTRAINT fk_mv_field_aggr FOREIGN KEY (aggr) REFERENCES wmd_aggregator_base (aggr_id),
  CONSTRAINT fk_mv_fielg_gpid FOREIGN KEY (field_group_id) REFERENCES pe_mv_field_group (field_group_id),
  CONSTRAINT ck_mv_field_idim CHECK (is_dimension IN ('Y', 'N')),
  CONSTRAINT ck_mv_field_ivir CHECK (is_virtual IN ('Y', 'N')),
  CONSTRAINT ck_mv_field_iava CHECK (is_available IN ('Y', 'N'))
);

CREATE UNIQUE INDEX pe_idx_mv_field_name ON pe_mv_field (mv_entity_id, UPPER(field_name));
CREATE SEQUENCE pe_seq_mv_field;

-- ------------------------------------------------------------------------
-- Temp table for inserting module version fields into pe_mv_field
-- ------------------------------------------------------------------------
CREATE GLOBAL TEMPORARY TABLE pe_mv_field_tmp
(
  entity_name                 VARCHAR2(100) NOT NULL,
  module_version_id           INTEGER NOT NULL,
  mv_entity_id                INTEGER,
  field_name                  VARCHAR2(128) NOT NULL,
  heading1                    VARCHAR2(40),
  heading2                    VARCHAR2(40),
  description                 VARCHAR2(1000),
  field_group_name            VARCHAR2(20),
  column_type_name            VARCHAR2(30) NOT NULL,
  is_dimension                CHAR(1) NOT NULL, 
  is_virtual                  CHAR(1) NOT NULL, 
  aggr_name                   VARCHAR2(20) NOT NULL,
  is_available                CHAR(1) NOT NULL,
  CONSTRAINT pk_mv_field_tmp PRIMARY KEY (entity_name, field_name)
) 
ON COMMIT DELETE ROWS;

-- ------------------------------------------------------------------------
-- Map module version field to field technology
-- ------------------------------------------------------------------------
CREATE TABLE pe_mv_field_tech_map
(
    mv_field_id  INTEGER NOT NULL,
    tech_id      INTEGER NOT NULL,
    CONSTRAINT pk_mv_field_tech_map PRIMARY KEY (mv_field_id, tech_id),
    CONSTRAINT fk_mv_field_tech_map_mfid FOREIGN KEY (mv_field_id) REFERENCES pe_mv_field(mv_field_id) ON DELETE CASCADE,
    CONSTRAINT fk_mv_field_tech_map_techid FOREIGN KEY (tech_id) REFERENCES pe_field_technology(tech_id)
);

-- ------------------------------------------------------------------------
-- Temp table to add new maps for module version field to field technology
-- ------------------------------------------------------------------------
CREATE GLOBAL TEMPORARY TABLE pe_mv_field_tech_map_tmp
(
  entity_name                 VARCHAR2(100) NOT NULL,
  module_version_id           INTEGER NOT NULL,
  mv_entity_id                INTEGER,
  field_name                  VARCHAR2(128) NOT NULL,
  tech_name                   VARCHAR2(8)   NOT NULL,
  CONSTRAINT pk_mv_field_tech_map_tmp PRIMARY KEY (entity_name, field_name, tech_name)
)
ON COMMIT DELETE ROWS;

-- ------------------------------------------------------------------------
-- Table to link module version fields to their imported attribute values
-- ------------------------------------------------------------------------
CREATE TABLE pe_mv_field_attribute_val
(
  mv_field_id             INTEGER NOT NULL,
  field_attribute_id      INTEGER NOT NULL,
  field_attribute_value   VARCHAR2(4000),
  CONSTRAINT pk_mv_field_attrval        PRIMARY KEY (mv_field_id, field_attribute_id), -- only one value per attribute
  CONSTRAINT fk_mv_field_attrval_fid    FOREIGN KEY (mv_field_id) REFERENCES pe_mv_field(mv_field_id) ON DELETE CASCADE,
  CONSTRAINT fk_mv_field_attrval_attrid FOREIGN KEY (field_attribute_id) REFERENCES pe_field_attribute(attribute_id) ON DELETE CASCADE
);


-- ------------------------------------------------------------------------
-- Temp table to update module version field attributes.
-- ------------------------------------------------------------------------
CREATE GLOBAL TEMPORARY TABLE pe_mv_field_attrval_tmp
(
  mv_entity_name    VARCHAR2(100),
  mv_field_name     VARCHAR2(128),
  attribute_name    VARCHAR2(40),
  attribute_value   VARCHAR2(4000),
  module_version_id INTEGER
)
ON COMMIT DELETE ROWS;

CREATE UNIQUE INDEX idx_mv_field_attrval_tmp ON pe_mv_field_attrval_tmp(UPPER(mv_entity_name), mv_field_name, UPPER(attribute_name));


-- ------------------------------------------------------------------------
-- Support for remote UDC fields.
-- ------------------------------------------------------------------------
CREATE TABLE pe_mv_udc_field
(
  mv_field_id                 INTEGER NOT NULL,
  datasource_id               INTEGER NOT NULL,
  field_name                  VARCHAR2(128) NOT NULL,
  heading1                    VARCHAR2(40),
  heading2                    VARCHAR2(40),
  description                 VARCHAR2(1000),
  udc_source                  VARCHAR2(4000),
  calc_source                 CLOB,
  owner                       VARCHAR2(64),
  modified                    DATE,
  original_datasource_id      INTEGER,
  column_type_id              INTEGER NOT NULL,
  aggr                        CHAR(1) NOT NULL,
  is_valid                    CHAR(1) DEFAULT 'N' NOT NULL,
  is_complex                  CHAR(1) DEFAULT 'N' NOT NULL,
  is_deleted                  CHAR(1) DEFAULT 'N' NOT NULL,
  CONSTRAINT pk_mv_udc_field_id PRIMARY KEY (mv_field_id, datasource_id),
  CONSTRAINT fk_mv_udc_field_ctid FOREIGN KEY (column_type_id) REFERENCES pe_column_type (column_type_id),
  CONSTRAINT fk_mv_udc_field_aggr FOREIGN KEY (aggr) REFERENCES wmd_aggregator_base (aggr_id),
  CONSTRAINT fk_mv_udc_field_valid CHECK (is_valid IN ('Y', 'N')),
  CONSTRAINT ck_mv_udc_field_complex CHECK(is_complex IN ('Y', 'N')),
  CONSTRAINT ck_mv_udc_field_deleted CHECK(is_deleted IN ('Y', 'N')),
  CONSTRAINT fk_mv_udc_field_fid  FOREIGN KEY (mv_field_id) REFERENCES pe_mv_field (mv_field_id) ON DELETE CASCADE,
  CONSTRAINT fk_mv_udc_field_dsid FOREIGN KEY (datasource_id) REFERENCES pe_datasource (datasource_id) ON DELETE CASCADE,
  CONSTRAINT fk_mv_udc_field_odsid FOREIGN KEY (original_datasource_id) REFERENCES pe_datasource (datasource_id)
) tablespace WM_FLEXPM;

CREATE INDEX pe_mv_udc_field_idx ON pe_mv_udc_field(mv_field_id);

-- ------------------------------------------------------------------------
-- Temp table for inserting module version udc fields into pe_mv_udc_field
-- ------------------------------------------------------------------------
CREATE GLOBAL TEMPORARY TABLE pe_mv_udc_field_tmp
(
  entity_name                 VARCHAR2(100) NOT NULL,
  module_version_id           INTEGER NOT NULL,
  mv_entity_id                INTEGER,
  field_name                  VARCHAR2(128) NOT NULL,
  heading1                    VARCHAR2(40),
  heading2                    VARCHAR2(40),
  description                 VARCHAR2(1000),
  field_group_name            VARCHAR2(20),
  column_type_name            VARCHAR2(30) NOT NULL,
  is_dimension                CHAR(1) NOT NULL, 
  is_virtual                  CHAR(1) NOT NULL, 
  aggr_name                   VARCHAR2(20) NOT NULL,
  original_datasource_name    VARCHAR2(100),
  udc_source                  VARCHAR2(4000),
  calc_source                 CLOB,
  owner                       VARCHAR2(64),
  modified                    DATE,
  is_valid                    CHAR(1),
  is_complex                  CHAR(1),
  CONSTRAINT pk_mv_udc_field_tmp PRIMARY KEY (entity_name, field_name)
) 
ON COMMIT DELETE ROWS;

-- ------------------------------------------------------------------------
-- Map module version field to field technology
-- ------------------------------------------------------------------------
CREATE TABLE pe_mv_udc_field_tech_map
(
    mv_field_id   INTEGER NOT NULL,
    datasource_id INTEGER NOT NULL,
    tech_id       INTEGER NOT NULL,
    CONSTRAINT pk_mv_udc_field_tech_map PRIMARY KEY (mv_field_id, datasource_id, tech_id),
    CONSTRAINT fk_mv_udc_field_tech_map_mfid FOREIGN KEY (mv_field_id) REFERENCES pe_mv_field(mv_field_id) ON DELETE CASCADE,
    CONSTRAINT fk_mv_udc_field_tech_map_dsid FOREIGN KEY (datasource_id) REFERENCES pe_datasource (datasource_id) ON DELETE CASCADE,
    CONSTRAINT fk_mv_udc_field_tech_map_tid FOREIGN KEY (tech_id) REFERENCES pe_field_technology(tech_id)
);

-- ------------------------------------------------------------------------
-- Table to link module version UDC fields to their imported attribute values
-- ------------------------------------------------------------------------
CREATE TABLE pe_mv_udc_field_attribute_val
(
  mv_field_id             INTEGER NOT NULL,
  datasource_id           INTEGER NOT NULL,
  field_attribute_id      INTEGER NOT NULL,
  field_attribute_value   VARCHAR2(4000),
  CONSTRAINT pk_mv_udc_field_attrval        PRIMARY KEY (mv_field_id, datasource_id, field_attribute_id), 
  CONSTRAINT fk_mv_udc_field_attrval_did    FOREIGN KEY (datasource_id) REFERENCES pe_datasource(datasource_id) ON DELETE CASCADE,
  CONSTRAINT fk_mv_udc_field_attrval_fid    FOREIGN KEY (mv_field_id) REFERENCES pe_mv_field(mv_field_id) ON DELETE CASCADE,
  CONSTRAINT fk_mv_udc_field_attrval_attrid FOREIGN KEY (field_attribute_id) REFERENCES pe_field_attribute(attribute_id) ON DELETE CASCADE
);


-- ------------------------------------------------------------------------
-- UDC dependency table
-- ------------------------------------------------------------------------
CREATE TABLE pe_mv_udc_field_depend
(
  module_version_id           INTEGER NOT NULL,
  parent_mv_field_id          INTEGER NOT NULL,
  child_mv_field_id           INTEGER NOT NULL,
  CONSTRAINT pk_mv_udc_field_depend PRIMARY KEY (module_version_id, parent_mv_field_id, child_mv_field_id),
  CONSTRAINT fk_mv_udc_field_depend_mvid FOREIGN KEY (module_version_id) REFERENCES pe_module_version (module_version_id) ON DELETE CASCADE,
  CONSTRAINT fk_mv_udc_field_depend_pfid FOREIGN KEY (parent_mv_field_id) REFERENCES pe_mv_field (mv_field_id),
  CONSTRAINT fk_mv_udc_field_depend_cfid FOREIGN KEY (child_mv_field_id) REFERENCES pe_mv_field (mv_field_id)
);

-- ------------------------------------------------------------------------
-- A field represents either a Measure or a Dimension (OLAP terms).
-- Measure. Represents an attribute of an entity, in
-- this case, a time-series attribute like 'dropped calls', 'attempts' etc.
-- This 'measure' needs to be mapped to a real attribute for each
-- module-version we intend to support. It's OK if some don't map, they
-- just show as null in the result set for that particular instance.
-- Dimension. In this case, a time-invariant property such as switch_name,
-- region, market, etc.
-- ------------------------------------------------------------------------
CREATE TABLE pe_field
(
  field_id              INTEGER NOT NULL,
  entity_id             INTEGER NOT NULL,
  ordinal               INTEGER NOT NULL,
  field_name            VARCHAR2(128) NOT NULL,
  aggr                  CHAR(1) NOT NULL,
  column_type_id        INTEGER NOT NULL,
  heading1              VARCHAR2(40),
  heading2              VARCHAR2(40),
  description           VARCHAR2(1000),
  is_enabled            CHAR(1) DEFAULT 'Y' NOT NULL,  -- Default enabled
  is_deleted            CHAR(1) DEFAULT 'N' NOT NULL,  -- Logically deleted
  is_mappable           CHAR(1) DEFAULT 'Y' NOT NULL, 
  CONSTRAINT pk_field   PRIMARY KEY (field_id),
  CONSTRAINT fk_field_eid FOREIGN KEY (entity_id) REFERENCES pe_entity (entity_id) ON DELETE CASCADE,
  CONSTRAINT fk_field_aggr FOREIGN KEY (aggr) REFERENCES wmd_aggregator_base (aggr_id),
  CONSTRAINT fk_field_ctid FOREIGN KEY (column_type_id) REFERENCES pe_column_type (column_type_id),
  CONSTRAINT ck_field_aggr CHECK (aggr != 'U'),
  CONSTRAINT ck_field_iena CHECK (is_enabled IN ('Y', 'N')),
  CONSTRAINT ck_field_idel CHECK (is_deleted IN ('Y', 'N')),
  CONSTRAINT ck_field_imap CHECK (is_mappable IN ('Y', 'N'))
);

CREATE SEQUENCE pe_seq_field;
CREATE UNIQUE INDEX pe_idx_unq_field_name ON pe_field(entity_id, UPPER(field_name));


-- ------------------------------------------------------------------------
-- Relationships between entities. The cardinality of the relationship is
-- implicitly defined based on the mapping of columns to key columns
-- specified by pe_relation_key
-- ------------------------------------------------------------------------
CREATE TABLE pe_relation
(
  relation_id            INTEGER NOT NULL,
  from_entity_id         INTEGER NOT NULL,
  to_entity_id           INTEGER NOT NULL,
  relation_name          VARCHAR2(30) NOT NULL,
  CONSTRAINT pk_relation PRIMARY KEY (relation_id),
  CONSTRAINT fk_relation_fpid FOREIGN KEY (from_entity_id) REFERENCES pe_entity (entity_id) ON DELETE CASCADE,
  CONSTRAINT fk_relation_tpid FOREIGN KEY (to_entity_id) REFERENCES pe_entity (entity_id) ON DELETE CASCADE  
);

CREATE SEQUENCE pe_seq_relation;

-- ------------------------------------------------------------------------
-- We don't use wmd_dict_table to store table names because of the FK
-- constraint to wmn_entity. We therefore have our own.
-- Table type T = Traffic Data, E = Entity Instance Data
-- ------------------------------------------------------------------------
CREATE TABLE pe_table
(
  table_id               INTEGER NOT NULL,
  table_name             VARCHAR2(30) NOT NULL,
  temp_table_name        VARCHAR2(30) NOT NULL,
  avail_table_name       VARCHAR2(30),
  table_type             CHAR(1) NOT NULL,
  entity_id              INTEGER NOT NULL,
  status                 CHAR(1) DEFAULT 'U' NOT NULL,
  CONSTRAINT pk_table PRIMARY KEY (table_id),
  CONSTRAINT fk_table_eid FOREIGN KEY (entity_id) REFERENCES pe_entity(entity_id) ON DELETE CASCADE,
  CONSTRAINT fk_table_stat FOREIGN KEY (status) REFERENCES pe_db_status (status),
  CONSTRAINT ck_table_type CHECK (table_type IN ('T', 'E'))
);

CREATE UNIQUE INDEX pe_idx_table_name ON pe_table (table_name);
CREATE INDEX pe_idx_table_eid ON pe_table(entity_id);
CREATE INDEX pe_idx_table_stat ON pe_table(status);
CREATE SEQUENCE pe_seq_table;

-- ------------------------------------------------------------------------
-- INSERT trigger on USER_TABLES. So no need to create an index
-- (probably).
-- ------------------------------------------------------------------------
CREATE OR REPLACE TRIGGER PE_TABLE_TR_BI
  BEFORE INSERT ON pe_table
  FOR EACH ROW
DECLARE
  tc INTEGER;
BEGIN
   IF (:NEW.table_name IS NOT NULL)
   THEN
      SELECT COUNT(*) INTO tc FROM user_tables 
  WHERE table_name = UPPER(:NEW.table_name);
   END IF;
  
   IF tc > 0 
   THEN
      RAISE_APPLICATION_ERROR(-20001, 'pe_table:table_name ' || :NEW.table_name || ' already exists');
   END IF;
END;
/
SHOW ERRORS;

-- ------------------------------------------------------------------------
-- Fields that are dimension fields also have records in this table so we
-- can map them directly onto a column in an entity table. This column
-- is always the same for all module versions, because we need to be able
-- to rely on these columns for the purpose of identity.
-- ------------------------------------------------------------------------
CREATE TABLE pe_dimension_field
(
  field_id                INTEGER NOT NULL, -- PK and FK of pe_field
  key_ordinal             INTEGER, -- position in PK, or null if N/A
  table_id                INTEGER, -- which entity table is this field in
  local_data              CHAR(1) NOT NULL, -- 'Y' if values are managed locally by admin tool
  status                  CHAR(1) DEFAULT 'U' NOT NULL,
  CONSTRAINT pk_dimension_field     PRIMARY KEY (field_id),
  CONSTRAINT fk_dimension_field_fid FOREIGN KEY (field_id) REFERENCES pe_field (field_id) ON DELETE CASCADE,
  CONSTRAINT fk_dimension_field_tid FOREIGN KEY (table_id) REFERENCES pe_table (table_id) ON DELETE CASCADE,
  CONSTRAINT fk_dimension_field_stat FOREIGN KEY (status) REFERENCES pe_db_status (status),
  CONSTRAINT ck_dimension_field_ldata CHECK (local_data IN ('Y', 'N'))
);

CREATE INDEX pe_idx_dimen_fld_tid ON pe_dimension_field(table_id);
CREATE INDEX pe_idx_dimen_fld_sid ON pe_dimension_field(status);


-- ------------------------------------------------------------------------
-- INSERT trigger on dimension fields. This prevents invalid dimension
-- fields from getting into the database.
-- ------------------------------------------------------------------------
CREATE OR REPLACE TRIGGER pe_dimension_field_tr_bi
  BEFORE INSERT OR UPDATE
  ON pe_dimension_field
  FOR EACH ROW
DECLARE
  tc INTEGER;
BEGIN

   IF (:NEW.field_id IS NOT NULL)
   THEN
      SELECT LENGTH(f.field_name) INTO tc
        FROM pe_field f
       WHERE f.field_id = :NEW.field_id;
   END IF;
  
   IF tc > 30
   THEN
      RAISE_APPLICATION_ERROR(-20001, 'Dimension field name must be less than 30 characters');
   END IF;
END;
/
SHOW ERRORS;

-- ------------------------------------------------------------------------
-- RelationKey. A relationship between two entities is defined by mapping
-- fields from the 'from_entity' to id columns in the 'to_entity'. If
-- not all id columns are mapped, the relationship is one-many.
-- ------------------------------------------------------------------------
CREATE TABLE pe_relation_key
(
  relation_id            INTEGER NOT NULL,
  key_ordinal            INTEGER NOT NULL,
  field_id               INTEGER NOT NULL,
  CONSTRAINT pk_relation_key PRIMARY KEY (relation_id, key_ordinal),
  CONSTRAINT fk_relation_key_rid FOREIGN KEY (relation_id) REFERENCES pe_relation (relation_id) ON DELETE CASCADE,
  CONSTRAINT fk_relation_key_fid FOREIGN KEY (field_id) REFERENCES pe_dimension_field (field_id) ON DELETE CASCADE
);



-- ------------------------------------------------------------------------
-- Each module field represents a measure that has been mapped to a 
-- particular module instance.
-- ------------------------------------------------------------------------
CREATE TABLE pe_mv_field_map
(
  mv_field_map_id        INTEGER NOT NULL,
  module_version_id      INTEGER NOT NULL,
  field_id               INTEGER NOT NULL,
  numerator              VARCHAR2(4000), -- as determined by 
  denominator            VARCHAR2(4000), -- DDSQL
  table_id               INTEGER,
  mv_field_id            INTEGER NOT NULL,
  mv_group_id            INTEGER,
  CONSTRAINT pk_mv_field_map  PRIMARY KEY (mv_field_map_id),
  CONSTRAINT fk_mv_field_map_mvid FOREIGN KEY (module_version_id) REFERENCES pe_module_version(module_version_id) ON DELETE CASCADE,
  CONSTRAINT fk_mv_field_map_fid FOREIGN KEY (field_id) REFERENCES pe_field(field_id) ON DELETE CASCADE,
  CONSTRAINT fk_mv_field_map_tid FOREIGN KEY (table_id) REFERENCES pe_table(table_id),
  CONSTRAINT fk_mv_field_map_mvfid FOREIGN KEY (mv_field_id) REFERENCES pe_mv_field(mv_field_id) ON DELETE CASCADE,
  CONSTRAINT fk_mv_field_map_mvgid FOREIGN KEY (mv_group_id) REFERENCES pe_module_version_group (module_version_group_id),
  CONSTRAINT chk_mv_group_id_null CHECK(mv_group_id IS NOT NULL)
);

CREATE INDEX pe_idx_mv_fld_map_mvid ON pe_mv_field_map(module_version_id);
CREATE INDEX pe_idx_mv_fld_map_fid ON pe_mv_field_map(field_id);
CREATE INDEX pe_idx_mv_fld_map_tid ON pe_mv_field_map(table_id);
CREATE INDEX pe_idx_mv_fld_map_mvfid ON pe_mv_field_map(mv_field_id);
CREATE INDEX pe_idx_mv_fld_map_mvgid ON pe_mv_field_map(mv_group_id);

CREATE SEQUENCE pe_seq_mv_field_map;

-- ------------------------------------------------------------------------
-- To support a single field, we might need to store several real counters,
-- depending on the expression generated by ddsql (or whatever the agent
-- uses to fetch the data from the remote server).
-- ------------------------------------------------------------------------
CREATE TABLE pe_stored_field
(
  stored_field_id        INTEGER NOT NULL,
  column_name            VARCHAR2(30) NOT NULL,
  column_type_id         INTEGER NOT NULL,
  table_id               INTEGER,
  component              VARCHAR2(2000) NOT NULL,
  status                 CHAR(1) DEFAULT 'U' NOT NULL,
  entity_id              INTEGER NOT NULL,
  CONSTRAINT pk_stored_field PRIMARY KEY (stored_field_id),
  CONSTRAINT fk_stored_field_tid FOREIGN KEY (table_id) REFERENCES pe_table(table_id) ON DELETE CASCADE,
  CONSTRAINT fk_stored_field_ctid FOREIGN KEY (column_type_id) REFERENCES pe_column_type(column_type_id),
  CONSTRAINT fk_stored_field_stat FOREIGN KEY (status) REFERENCES pe_db_status (status),
  CONSTRAINT fk_stored_field_entid FOREIGN KEY (entity_id) REFERENCES pe_entity ( entity_id ) ON DELETE CASCADE
);

CREATE INDEX pe_idx_stored_fld_tid ON pe_stored_field(table_id);
CREATE INDEX pe_idx_stored_fld_ctid ON pe_stored_field(column_type_id);
CREATE INDEX pe_idx_stored_fld_stat ON pe_stored_field(status);
CREATE INDEX pe_idx_stored_fld_eid ON pe_stored_field(entity_id);

-- ------------------------------------------------------------------------
--
--
-- ------------------------------------------------------------------------
CREATE TABLE pe_dropped_columns
(
  table_id         INTEGER NOT NULL,
  column_name      VARCHAR2(30) NOT NULL,
  column_type_id   INTEGER NOT NULL,
  drop_date        DATE    DEFAULT TRUNC( SYSDATE, 'DD' ) NOT NULL,
    CONSTRAINT pk_pe_dropped_columns PRIMARY KEY ( table_id, column_name ),
    CONSTRAINT fk_pe_dropped_columns_tid FOREIGN KEY (table_id) REFERENCES pe_table(table_id) ON DELETE CASCADE,
    CONSTRAINT fk_pe_dropped_columns_ctid FOREIGN KEY (column_type_id) REFERENCES pe_column_type(column_type_id)
);

CREATE INDEX pe_idx_dropped_col_tid ON pe_dropped_columns(table_id);
CREATE INDEX pe_idx_dropped_col_ctid ON pe_dropped_columns(column_type_id);

CREATE SEQUENCE pe_seq_dropped_column_name;

-- ------------------------------------------------------------------------
-- How should we implement column uniqueness? INDEX or TRIGGER?
-- ------------------------------------------------------------------------
CREATE UNIQUE INDEX pe_idx_stored_field_tcn ON pe_stored_field(entity_id, column_name);

CREATE SEQUENCE pe_seq_stored_field;


-- ------------------------------------------------------------------------
-- This table is used to map module version fields onto real fields in
-- either an entity or traffic table in the traffic cache. It is populated
-- (rather, maintained) by agents that manipulate the traffic cache based
-- on enterprise equivalency mappings.
-- ------------------------------------------------------------------------
CREATE TABLE pe_stored_field_map
(
  stored_field_id        INTEGER NOT NULL,
  mv_field_map_id        INTEGER NOT NULL,
  CONSTRAINT pk_stored_field_map  PRIMARY KEY (stored_field_id, mv_field_map_id),
  CONSTRAINT fk_stored_field_map_sfid FOREIGN KEY (stored_field_id) REFERENCES pe_stored_field(stored_field_id) ON DELETE CASCADE,
  CONSTRAINT fk_stored_field_map_mvfmid FOREIGN KEY (mv_field_map_id) REFERENCES pe_mv_field_map(mv_field_map_id) ON DELETE CASCADE
);

CREATE INDEX pe_idx_stored_fld_map_sfid ON pe_stored_field_map(stored_field_id);
CREATE INDEX pe_idx_stored_fld_map_mvfmid ON pe_stored_field_map(mv_field_map_id);

-- ------------------------------------------------------------------------
-- Need somewhere to record which stored fields are required for which
-- module instances. This allows us to identify stored fields that are 
-- no longer required so we can reuse the column for the next mapping
-- that needs a column of this type.
-- ------------------------------------------------------------------------
CREATE TABLE pe_field_instance_map
(
  stored_field_id        INTEGER NOT NULL,
  module_instance_id     INTEGER NOT NULL,
  CONSTRAINT fk_field_instance_map_fid FOREIGN KEY (stored_field_id) REFERENCES pe_stored_field(stored_field_id) ON DELETE CASCADE,
  CONSTRAINT fk_field_instance_map_mid FOREIGN KEY (module_instance_id) REFERENCES pe_module_instance(module_instance_id) ON DELETE CASCADE
);

-- ------------------------------------------------------------------------
-- stores what file types are loaded in each module version.
-- populated by the data dictionary agent.
-- used by the data availability agent.
-- ------------------------------------------------------------------------
CREATE TABLE pe_mv_file_type
(
   mv_file_type_id   INTEGER NOT NULL,
   module_version_id INTEGER NOT NULL,
   mv_file_type_name VARCHAR2(30) NOT NULL,   
   frequency_id      INTEGER NOT NULL,
   mv_entity_id      INTEGER NOT NULL, -- top entity for the file load
   CONSTRAINT pk_pe_mv_file_type PRIMARY KEY(mv_file_type_id),   
   CONSTRAINT fk_pe_mv_file_type_mvid FOREIGN KEY(module_version_id) REFERENCES pe_module_version(module_version_id) ON DELETE CASCADE,
   CONSTRAINT fk_pe_mv_file_type_freq FOREIGN KEY(frequency_id) REFERENCES wm_frequency(frequency_id) ON DELETE CASCADE,
   CONSTRAINT fk_pe_mv_file_type_mveid  FOREIGN KEY(mv_entity_id) REFERENCES pe_mv_entity(mv_entity_id) ON DELETE CASCADE
);

CREATE UNIQUE INDEX pe_idx_mv_file_type_name ON pe_mv_file_type(module_version_id, mv_file_type_name);

CREATE SEQUENCE pe_seq_mv_file_type;

-- ------------------------------------------------------------------------
-- stores a map which tells for each module version entity, what
-- file types load it.
-- ------------------------------------------------------------------------
CREATE TABLE pe_mv_entity_file_type
(
   mv_entity_id      INTEGER NOT NULL,
   mv_file_type_id   INTEGER NOT NULL,
   CONSTRAINT pk_pe_mv_entity_file_type PRIMARY KEY(mv_entity_id, mv_file_type_id),
   CONSTRAINT fk_pe_mv_entity_mveid  FOREIGN KEY(mv_entity_id) REFERENCES pe_mv_entity(mv_entity_id) ON DELETE CASCADE,
   CONSTRAINT fk_pe_mv_entity_file_type FOREIGN KEY(mv_file_type_id) REFERENCES pe_mv_file_type(mv_file_type_id) ON DELETE CASCADE
);

-- ------------------------------------------------------------------------
-- helps populate pe_mv_file_type and pe_mv_entity_file_type
-- ------------------------------------------------------------------------
CREATE GLOBAL TEMPORARY TABLE pe_mv_file_type_tmp
(
   remote_entity_name     VARCHAR2(100) NOT NULL,
   remote_file_type       VARCHAR2(30)  NOT NULL,
   remote_frequency_id    INTEGER NOT NULL,
   remote_top_entity_name VARCHAR2(100) NOT NULL
) ON COMMIT DELETE ROWS;   

-- ------------------------------------------------------------------------
-- stores status of events looked at in various datasources:
-- ------------------------------------------------------------------------
CREATE TABLE pe_remote_data_load_event
(
   datasource_id INTEGER NOT NULL,
   entity_id     INTEGER NOT NULL,
   event_id      INTEGER NOT NULL,
   event_time    DATE NOT NULL,
   data_tstamp   DATE NOT NULL,
   file_type     VARCHAR2(30),
   CONSTRAINT pk_remote_data_load_event PRIMARY KEY(datasource_id, entity_id),
   CONSTRAINT fk_remote_data_load_event_ds FOREIGN KEY (datasource_id) REFERENCES pe_datasource(datasource_id) ON DELETE CASCADE,
   CONSTRAINT fk_remote_data_load_event_e FOREIGN KEY (entity_id) REFERENCES pe_entity(entity_id) ON DELETE CASCADE
);

CREATE INDEX pe_idx_remote_dla_eid ON pe_remote_data_load_event(entity_id);


-- This version used for Enterprise reports. It is much more efficient than
-- rg_busyhour_tmp, because it does not have an index. This makes inserts
-- into it over twice as fast. See SEAde44666. We make it session-based, so
-- we can use DIRECT LOAD (see INSERT /*+ APPEND */ INTO ...). Direct load
-- requires a COMMIT before you can read the data from an object modified 
-- in parallel or direct load.
CREATE GLOBAL TEMPORARY TABLE rg_driver_tmp
(
  instance_id INTEGER       NOT NULL,
  tstamp      DATE          NOT NULL,
  rank        INTEGER,
  VALUE       INTEGER
)
  ON COMMIT PRESERVE ROWS;

    
-- ------------------------------------------------------------------------
-- stores last rename history entry for remote data source:
-- ------------------------------------------------------------------------
CREATE TABLE pe_remote_udc_rename
(
  datasource_id          INTEGER NOT NULL,
  history_ID             INTEGER NOT NULL,
  tstamp                 DATE    NOT NULL,
  CONSTRAINT fk_rename_ds_id FOREIGN KEY (datasource_id) REFERENCES pe_datasource(datasource_id) ON DELETE CASCADE,
  CONSTRAINT pk_remote_udc_rename_ds_id   PRIMARY KEY (datasource_id)
);

-- ------------------------------------------------------------------------
-- Support for enterprise UDC fields.
-- ------------------------------------------------------------------------
CREATE TABLE pe_udc_field
(
  field_id                    INTEGER NOT NULL,
  udc_source                  VARCHAR2(4000) NOT NULL,
  calc_source                 CLOB,
  error_msg                   VARCHAR2(2000),
  owner                       VARCHAR2(64),
  modified                    DATE NOT NULL,
  is_valid                    CHAR(1) DEFAULT 'N' NOT NULL,
  is_complex                  CHAR(1) DEFAULT 'N' NOT NULL,
  CONSTRAINT pk_udc_field_id PRIMARY KEY (field_id),
  CONSTRAINT fk_udc_field_valid CHECK (is_valid IN ('Y', 'N')),
  CONSTRAINT ck_udc_field_complex CHECK(is_complex IN ('Y', 'N')),
  CONSTRAINT fk_udc_field_fid  FOREIGN KEY (field_id) REFERENCES pe_field (field_id) ON DELETE CASCADE
) tablespace WM_FLEXPM;

CREATE SEQUENCE pe_udc_field_seq;

-- ------------------------------------------------------------------------
-- Table to add a relationship between two entities not related via 
-- parent child relationships
-- ------------------------------------------------------------------------
CREATE TABLE pe_entity_relation
(
  primary_field_id    INTEGER NOT NULL,
  secondary_field_id  INTEGER NOT NULL,
  CONSTRAINT fk_entity_rel_pri FOREIGN KEY(primary_field_id) REFERENCES pe_field(field_id) ON DELETE CASCADE,
  CONSTRAINT fk_entity_rel_sec FOREIGN KEY(secondary_field_id) REFERENCES pe_field(field_id) ON DELETE CASCADE,
  CONSTRAINT ck_entity_rel_field CHECK(primary_field_id != secondary_field_id)
);

CREATE UNIQUE INDEX idx_entity_relation 
   ON pe_entity_relation(primary_field_id, secondary_field_id);



CREATE TABLE pe_entity_filetype
(
  datasource_id  INTEGER      NOT NULL,
  entity_id      INTEGER      NOT NULL,
  file_type_name VARCHAR2(30) NOT NULL,
  CONSTRAINT pk_pe_entity_filetype PRIMARY KEY (entity_id, datasource_id, file_type_name),
  CONSTRAINT fk_pe_entity_filetype_entid FOREIGN KEY (entity_id)
  REFERENCES pe_entity(entity_id) ON DELETE CASCADE,
  CONSTRAINT fk_pe_entity_filetype_dsid FOREIGN KEY (datasource_id)
  REFERENCES pe_datasource(datasource_id) ON DELETE CASCADE
)
ORGANIZATION INDEX
;

--
-- This table is used to cache which datasource_id/file_type_id pairs have 
-- entity-file type data already cached (in pe_entity_filetype). It is
-- possible to have a row in here, but nothing corresponding in the cache
-- if no mappings are present for this file-type.
-- This table needs to be cleared any time the mapping is changed for
-- anything on a datasource.
--
CREATE TABLE pe_datasource_filetype
(
  datasource_id  INTEGER      NOT NULL,
  file_type_id   INTEGER      NOT NULL,
  CONSTRAINT pk_pe_datasource_filetype PRIMARY KEY (datasource_id, file_type_id),
  CONSTRAINT fk_pe_datasource_filetype_dsid FOREIGN KEY (datasource_id)
  REFERENCES pe_datasource(datasource_id) ON DELETE CASCADE
)
ORGANIZATION INDEX;
   

CREATE OR REPLACE TRIGGER pe_mv_field_map_dsft_tr
  AFTER INSERT OR UPDATE OR DELETE ON pe_mv_field_map
DECLARE
BEGIN
  -- This triggers the data load event handler to
  -- recompute the filetype to pe_entity
  -- dependencies (which are themselves stored
  -- in pe_entity_filetype).
  DELETE FROM pe_datasource_filetype;
END;
/   
   
-- ----------------------------------------------------------------------------
--
-- Local Variables:
-- mode: pls
-- End:
